@@ -1157,65 +1157,62 @@ ConstructPolyline (sciPointObj * pparentsubwin, double *pvecx, double *pvecy, do
 
       pPOLYLINE_FEATURE (pobj)->isselected = TRUE;
       ppoly = pPOLYLINE_FEATURE (pobj);
-      if ((ppoly->pvector = MALLOC (n1 * sizeof (POINT2D))) == NULL)
-	{
-	  sciDelThisToItsParent (pobj, sciGetParent (pobj));
-	  sciDelHandle (pobj);
-	  FREE(pPOLYLINE_FEATURE(pobj));
-	  FREE(pobj);
-	  return (sciPointObj *) NULL;
-	}
+      
       /* pour le moment je garde les vecteurs separes, et non en POINT2D */
-      if ((ppoly->pvx = MALLOC (n1 * sizeof (double))) == NULL)
+      if ( n1 != 0 )
+      {
+        if ((ppoly->pvx = MALLOC (n1 * sizeof (double))) == NULL)
 	{
-	  FREE(pPOLYLINE_FEATURE (pobj)->pvector);
-	  sciDelThisToItsParent (pobj, sciGetParent (pobj));
+          sciDelThisToItsParent (pobj, sciGetParent (pobj));
 	  sciDelHandle (pobj);
 	  FREE(pPOLYLINE_FEATURE(pobj));
 	  FREE(pobj);
 	  return (sciPointObj *) NULL;
 	}
-      if ((ppoly->pvy = MALLOC (n1 * sizeof (double))) == NULL)
+        if ((ppoly->pvy = MALLOC (n1 * sizeof (double))) == NULL)
 	{
 	  FREE(pPOLYLINE_FEATURE (pobj)->pvx);
-	  FREE(pPOLYLINE_FEATURE (pobj)->pvector);
 	  sciDelThisToItsParent (pobj, sciGetParent (pobj));
 	  sciDelHandle (pobj);
 	  FREE(pPOLYLINE_FEATURE(pobj));
 	  FREE(pobj);
 	  return (sciPointObj *) NULL;
  	}
-
-      for (i = 0; i < n1; i++)
-	{
-	  ppoly->pvector[i].x = pvecx[i];
-	  ppoly->pvector[i].y = pvecy[i];
-	  ppoly->pvx[i]       = pvecx[i];
-	  ppoly->pvy[i]       = pvecy[i];
-	  
-	}
-
-      /**DJ.Abdemouche 2003**/
-      if (pvecz == (double *) NULL)
-	{
-	  pPOLYLINE_FEATURE (pobj)->pvz=NULL;
-	}
-      else
-	{
-	  if ((ppoly->pvz = MALLOC (n1 * sizeof (double))) == NULL)
-	    {
-	      FREE(pPOLYLINE_FEATURE (pobj)->pvx);
-	      FREE(pPOLYLINE_FEATURE (pobj)->pvector);
-	      FREE(pPOLYLINE_FEATURE (pobj)->pvy);
-	      sciDelThisToItsParent (pobj, sciGetParent (pobj));
-	      sciDelHandle (pobj);
-	      FREE(pPOLYLINE_FEATURE(pobj));
-	      FREE(pobj);
-	      return (sciPointObj *) NULL;
-	    } 
-	  for (i = 0; i < n1; i++)
+        
+        for (i = 0; i < n1; i++)
+        {
+          ppoly->pvx[i] = pvecx[i];
+          ppoly->pvy[i] = pvecy[i]; 
+        }
+        /**DJ.Abdemouche 2003**/
+        if ( pvecz == (double *) NULL )
+        {
+          pPOLYLINE_FEATURE (pobj)->pvz=NULL;
+        }
+        else
+        {
+          if ((ppoly->pvz = MALLOC (n1 * sizeof (double))) == NULL)
+          {
+            FREE(pPOLYLINE_FEATURE (pobj)->pvx);
+            FREE(pPOLYLINE_FEATURE (pobj)->pvy);
+            sciDelThisToItsParent (pobj, sciGetParent (pobj));
+            sciDelHandle (pobj);
+            FREE(pPOLYLINE_FEATURE(pobj));
+            FREE(pobj);
+            return (sciPointObj *) NULL;
+          } 
+          for (i = 0; i < n1; i++)
 	    ppoly->pvz[i] = pvecz[i];
-	}
+        }
+      }
+      else
+      {
+        ppoly->pvx = NULL ;
+        ppoly->pvy = NULL ;
+        ppoly->pvz = NULL ;
+      }
+      
+      
 
       ppoly->n1 = n1;	  /* memorisation du nombre de points */
       ppoly->n2 = n2;	  /* memorisation du nombre des courbes */
@@ -1226,7 +1223,6 @@ ConstructPolyline (sciPointObj * pparentsubwin, double *pvecx, double *pvecy, do
 	{
 	  FREE(pPOLYLINE_FEATURE (pobj)->pvy);
 	  FREE(pPOLYLINE_FEATURE (pobj)->pvx);
-	  FREE(pPOLYLINE_FEATURE (pobj)->pvector);
 	  sciDelThisToItsParent (pobj, sciGetParent (pobj));
 	  sciDelHandle (pobj);
 	  FREE(pPOLYLINE_FEATURE(pobj));
@@ -1390,6 +1386,12 @@ ConstructRectangle (sciPointObj * pparentsubwin, double x, double y,
 {
   sciPointObj *pobj = (sciPointObj *) NULL; 
  
+  if ( height < 0.0 || width < 0.0 )
+  {
+    Scierror(999,"Width and height must be positive.\n");
+    return NULL ;
+  }
+
   if (sciGetEntityType (pparentsubwin) == SCI_SUBWIN)
     {
       if ((pobj = MALLOC ((sizeof (sciPointObj)))) == NULL)
@@ -1763,7 +1765,6 @@ ConstructSurface (sciPointObj * pparentsubwin, sciTypeOf3D typeof3d,
       psurf->dimzx = dimzx; /* dimzx is completly equal to m3*/
       psurf->dimzy = dimzy; /* dimzx is completly equal to n3*/
       psurf->izcol = izc;
-      psurf->pproj = NULL;	/* Les projections ne sont pas encore calculees */
       psurf->isselected = TRUE;
 
       psurf->flag[0] = flag[0]; /* F.Leray 16.04.04 HERE We store the flag=[mode (hidden part ), type (scaling), box (frame around the plot)] */
