@@ -7,10 +7,10 @@
  *    Fabrice Leray,     INRIA 2004-xxxx
  *    Comment:
  *    This file contains all functions used to BUILD new objects : 
-        - break the binding between the deleted object and its parent in the 
-          existing hierarchy
-        - freeing memory
-  --------------------------------------------------------------------------*/
+ - break the binding between the deleted object and its parent in the 
+ existing hierarchy
+ - freeing memory
+ --------------------------------------------------------------------------*/
 
 #include <stdio.h> 
 #include <string.h>
@@ -25,6 +25,12 @@
 #include "SetProperty.h"
 #include "Interaction.h" /* for callback funtions */
 
+#if WIN32
+#include "../os_specific/win_mem_alloc.h" /* MALLOC */
+#else
+#include "../os_specific/sci_mem_alloc.h" /* MALLOC */
+#endif
+
 
 /********************* modifier le 01/02/2002 ************************
  * On detruit pas la sous fenetre, elle est initialiser avec la figure
@@ -127,7 +133,7 @@ DestroyAllGraphicsSons (sciPointObj * pthis)
       return 0;
       break;
     case SCI_AGREG:
-      DestroyAgregation (pthis);
+      DestroyCompound (pthis);
       return 0;
       break; 
     case SCI_MERGE:
@@ -138,6 +144,10 @@ DestroyAllGraphicsSons (sciPointObj * pthis)
       DestroyLabel (pthis);
       return 0;
       break;
+    case SCI_UIMENU:
+      DestroyUimenu (pthis);
+      return 0;
+      break;
     default:
       sciprint ("Entity with type %d cannot be destroyed\n",sciGetEntityType (pthis));
       return -1;
@@ -177,6 +187,7 @@ sciDelGraphicObj (sciPointObj * pthis)
     case SCI_TEXT:
     case SCI_MERGE: 
     case SCI_LABEL:
+    case SCI_UIMENU:
       DestroyAllGraphicsSons (pthis);
       return 0;
       break;
@@ -219,6 +230,8 @@ DestroyFigure (sciPointObj * pthis)
    */
   FREE ((sciGetFontContext(pthis))->pfontname);
   FREE(pFIGURE_FEATURE(pthis)->pcolormap);
+  FREE(pFIGURE_FEATURE(pthis)->user_data);
+  pFIGURE_FEATURE(pthis)->size_of_user_data = 0;
   FREE (sciGetPointerToFeature (pthis));
   FREE (pthis);
   /* delete windows() */
@@ -279,6 +292,9 @@ DestroySubWin (sciPointObj * pthis)
   ppsubwin->axes.nygrads = 0;
   ppsubwin->axes.nzgrads = 0;
 
+  FREE(ppsubwin->user_data);
+  ppsubwin->size_of_user_data = 0;
+
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
@@ -336,6 +352,9 @@ int
 DestroyText (sciPointObj * pthis)
 {
   FREE (pTEXT_FEATURE (pthis)->ptextstring);
+  FREE (pTEXT_FEATURE (pthis)->user_data);
+  pTEXT_FEATURE (pthis)->size_of_user_data = 0;
+  
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
@@ -377,8 +396,12 @@ DestroyTitle (sciPointObj * pthis)
 int
 DestroyLegend (sciPointObj * pthis)
 {
-  FREE(pLEGEND_FEATURE (pthis)->pptabofpointobj);
-  FREE (pLEGEND_FEATURE (pthis)->text.ptextstring);
+  sciLegend * ppLegend = pLEGEND_FEATURE (pthis) ;
+  FREE(ppLegend->pptabofpointobj);
+  FREE (ppLegend->text.ptextstring);
+  FREE (ppLegend->user_data);
+  ppLegend->size_of_user_data = 0;
+
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
@@ -398,9 +421,11 @@ DestroyLegend (sciPointObj * pthis)
 int
 DestroyPolyline (sciPointObj * pthis)
 {
-  FREE (pPOLYLINE_FEATURE (pthis)->pvector);
   FREE (pPOLYLINE_FEATURE (pthis)->pvx);
   FREE (pPOLYLINE_FEATURE (pthis)->pvy);
+  FREE (pPOLYLINE_FEATURE (pthis)->user_data);
+  pPOLYLINE_FEATURE (pthis)->size_of_user_data = 0;
+
   if (pPOLYLINE_FEATURE (pthis)->pvz != NULL) /**DJ.Abdemouche 2003**/
     FREE (pPOLYLINE_FEATURE (pthis)->pvz);
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
@@ -420,6 +445,8 @@ DestroyPolyline (sciPointObj * pthis)
 int
 DestroyArc (sciPointObj * pthis)
 {
+  FREE (pARC_FEATURE (pthis)->user_data);
+  pARC_FEATURE (pthis)->size_of_user_data = 0;
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
@@ -439,6 +466,8 @@ DestroyArc (sciPointObj * pthis)
 int
 DestroyRectangle (sciPointObj * pthis)
 {
+  FREE (pRECTANGLE_FEATURE (pthis)->user_data);
+  pRECTANGLE_FEATURE (pthis)->size_of_user_data = 0;
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
@@ -462,6 +491,8 @@ DestroySurface (sciPointObj * pthis)
   integer cmpt;
   
   psubwin = (sciPointObj *) sciGetParentSubwin(pthis);
+  FREE (pSURFACE_FEATURE (pthis)->user_data);
+  pSURFACE_FEATURE (pthis)->size_of_user_data = 0;
   FREE(pSURFACE_FEATURE (pthis)->pvecz);
   FREE(pSURFACE_FEATURE (pthis)->pvecy);
   FREE(pSURFACE_FEATURE (pthis)->pvecx);
@@ -475,7 +506,7 @@ DestroySurface (sciPointObj * pthis)
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
-  /*FREE (pSURFACE_FEATURE (pthis)->pproj);*/
+
   FREE (sciGetPointerToFeature (pthis));
   FREE (pthis);
   cmpt=0;
@@ -501,6 +532,8 @@ DestroySurface (sciPointObj * pthis)
 int DestroyMerge (sciPointObj * pthis)
 {
   pSUBWIN_FEATURE (sciGetParentSubwin(pthis))->facetmerge = FALSE;
+  FREE (pMERGE_FEATURE (pthis)->user_data);
+  pMERGE_FEATURE (pthis)->size_of_user_data = 0;
   FREE(pMERGE_FEATURE (pthis)->index_in_entity);
   FREE(pMERGE_FEATURE (pthis)->from_entity);
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
@@ -522,6 +555,8 @@ int
 DestroyGrayplot (sciPointObj * pthis)
 {
   FREE (pGRAYPLOT_FEATURE (pthis)->pvecx);
+  FREE (pGRAYPLOT_FEATURE (pthis)->user_data);
+  pGRAYPLOT_FEATURE (pthis)->size_of_user_data = 0;
   if (pGRAYPLOT_FEATURE (pthis)->type != 2)
     FREE (pGRAYPLOT_FEATURE (pthis)->pvecy);
   FREE (pGRAYPLOT_FEATURE (pthis)->pvecz);
@@ -543,6 +578,9 @@ DestroyAxes (sciPointObj * pthis)
 {
   int i;
   char **str;
+
+  FREE (pAXES_FEATURE (pthis)->user_data);
+  pAXES_FEATURE (pthis)->size_of_user_data = 0;
   
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
@@ -569,6 +607,8 @@ DestroyAxes (sciPointObj * pthis)
 int
 DestroyFec (sciPointObj * pthis)
 {
+  FREE (pFEC_FEATURE (pthis)->user_data);
+  pFEC_FEATURE (pthis)->size_of_user_data = 0;
   FREE (pFEC_FEATURE (pthis)->pvecx);
   FREE (pFEC_FEATURE (pthis)->pvecy);
   FREE (pFEC_FEATURE (pthis)->pnoeud); 
@@ -597,6 +637,8 @@ int
 DestroySegs (sciPointObj * pthis)
 {  
   
+  FREE (pSEGS_FEATURE (pthis)->user_data);
+  pSEGS_FEATURE (pthis)->size_of_user_data = 0;
   FREE (pSEGS_FEATURE (pthis)->vx);
   FREE (pSEGS_FEATURE (pthis)->vy); 
   if (pSEGS_FEATURE (pthis)->vz != (double *)NULL) 
@@ -607,9 +649,9 @@ DestroySegs (sciPointObj * pthis)
     } 
   else 
     {
-	FREE(pSEGS_FEATURE (pthis)->vfx); pSEGS_FEATURE (pthis)->vfx = NULL;
-	FREE(pSEGS_FEATURE (pthis)->vfy); pSEGS_FEATURE (pthis)->vfy = NULL;
-	FREE(pSEGS_FEATURE (pthis)->vfz); pSEGS_FEATURE (pthis)->vfz = NULL;
+      FREE(pSEGS_FEATURE (pthis)->vfx); pSEGS_FEATURE (pthis)->vfx = NULL;
+      FREE(pSEGS_FEATURE (pthis)->vfy); pSEGS_FEATURE (pthis)->vfy = NULL;
+      FREE(pSEGS_FEATURE (pthis)->vfz); pSEGS_FEATURE (pthis)->vfz = NULL;
     } 
   sciDelThisToItsParent (pthis, sciGetParent (pthis)); 
   if (sciDelHandle (pthis) == -1)
@@ -620,12 +662,17 @@ DestroySegs (sciPointObj * pthis)
   return 0;
 }
 
-/**DestroyAgregation
- * @memo This function destroies the agregation and the elementaries structures and only this to destroy all sons use DelGraphicsSon
+/**DestroyCompound
+ * @memo This function destroies the Compound and the elementaries structures and only this to destroy all sons use DelGraphicsSon
  */
 int
-DestroyAgregation (sciPointObj * pthis)
+DestroyCompound (sciPointObj * pthis)
 {
+  sciAgreg * ppagreg = pAGREG_FEATURE (pthis);
+
+  FREE (ppagreg->user_data);
+  pAGREG_FEATURE (pthis)->size_of_user_data = 0;
+
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1) return -1;
   FREE (sciGetPointerToFeature (pthis));
@@ -649,11 +696,11 @@ void DeleteObjs(integer win_num)
 }
 
 
-/**sciUnAgregation
- * @memo This function destroies the agregation and  and unpackes the elementaries structures to associates them to its parent
+/**sciUnCompound
+ * @memo This function destroies the Compound and  and unpackes the elementaries structures to associates them to its parent
  */
 int
-sciUnAgregation (sciPointObj * pobj)
+sciUnCompound (sciPointObj * pobj)
 {
   sciPointObj *pparent, *pobjson;
   sciSons *psons = (sciSons *)NULL;
@@ -668,15 +715,16 @@ sciUnAgregation (sciPointObj * pobj)
     {
       /* we delete this son to this */ 
       pobjson = psons->pointobj;
+      /* take the previous sons before the current is freed */
+      psons = psons->pprev;
       sciDelThisToItsParent (pobjson, pobj);
       /* and link to its old parent */
       sciAddThisToItsParent (pobjson, pparent);
-      psons = psons->pprev;
     }
 
   sciSetCurrentObj(pparent); /* pparent is the new current object */
 
-  DestroyAgregation(pobj);
+  DestroyCompound(pobj);
   return 0;
 }
 
@@ -685,10 +733,12 @@ sciUnAgregation (sciPointObj * pobj)
  * @memo This function destroies the Subwindow (the Axe) and the elementaries structures and only this to destroy all sons use DelGraphicsSon
  * @param sciPointObj * pthis: the pointer to the entity
  */
-int
-DestroyLabel (sciPointObj * pthis)
+int DestroyLabel (sciPointObj * pthis)
 {
-  FREE (pLABEL_FEATURE (pthis)->text.ptextstring);
+  sciLabel * ppLabel = pLABEL_FEATURE (pthis);
+  FREE (ppLabel->user_data); 
+  ppLabel->size_of_user_data = 0;
+  FREE (ppLabel->text.ptextstring);
   sciDelThisToItsParent (pthis, sciGetParent (pthis));
   if (sciDelHandle (pthis) == -1)
     return -1;
@@ -699,6 +749,19 @@ DestroyLabel (sciPointObj * pthis)
   return 0;
 }
 
+int DestroyUimenu (sciPointObj * pthis)
+{
+  FREE (pUIMENU_FEATURE (pthis)->label.ptextstring);
+  FREE (pUIMENU_FEATURE (pthis)->label.callback);
+
+  sciDelThisToItsParent (pthis, sciGetParent (pthis));
+  if (sciDelHandle (pthis) == -1) return -1;
+  FREE (sciGetPointerToFeature (pthis));
+  FREE (pthis);
+  /* on peut alors destroyer le parent */
+  return 0;
+}
+
 
 
 /**del sciDelLabelsMenu
