@@ -23,6 +23,13 @@
 #include "bcg.h"
 #include "PloEch.h"
 
+#if WIN32
+#include "../os_specific/win_mem_alloc.h" /* MALLOC */
+#else
+#include "../os_specific/sci_mem_alloc.h" /* MALLOC */
+#endif
+
+
 sciPointObj *pfiguremdl = (sciPointObj *) NULL;
 sciPointObj *paxesmdl = (sciPointObj *) NULL;
 
@@ -41,7 +48,7 @@ int C2F(graphicsmodels) ()
   sciSubWindow * ppobj = NULL;
   double tab[] = {0.,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.}; /* graduations init. tmptab */
   sciSubWindow * ppaxesmdl = NULL;
- /*  sciSubWindow * ppaxesmdl = NULL; */
+  /*  sciSubWindow * ppaxesmdl = NULL; */
 
   if ((pfiguremdl = MALLOC ((sizeof (sciPointObj)))) == NULL)
     {
@@ -79,6 +86,8 @@ int C2F(graphicsmodels) ()
   
   sciSetCurrentSon (pfiguremdl, (sciPointObj *) NULL);
 
+  pFIGURE_FEATURE (pfiguremdl)->user_data = (int *) NULL; /* adding 30.06.05 */
+  pFIGURE_FEATURE (pfiguremdl)->size_of_user_data = 0;
   pFIGURE_FEATURE (pfiguremdl)->relationship.psons = (sciSons *) NULL;
   pFIGURE_FEATURE (pfiguremdl)->relationship.plastsons = (sciSons *) NULL;
   pFIGURE_FEATURE (pfiguremdl)->numcolors=  NUMCOLORS_SCI;
@@ -130,7 +139,6 @@ int C2F(graphicsmodels) ()
     }
 
   pFIGURE_FEATURE (pfiguremdl)->allredraw = FALSE;
-  
   strncpy (pFIGURE_FEATURE (pfiguremdl)->name, "Scilab Graphic (%d)", sizeof ("Scilab Graphic (%d)") + 4);
   pFIGURE_FEATURE (pfiguremdl)->namelen = Min (60, 19); 
   pFIGURE_FEATURE (pfiguremdl)->number=0;
@@ -186,7 +194,11 @@ int C2F(graphicsmodels) ()
       return 0;
     }
   sciSetCurrentSon (paxesmdl, (sciPointObj *) NULL);
+  
   ppaxesmdl =  pSUBWIN_FEATURE (paxesmdl);
+
+  ppaxesmdl->user_data = (int *) NULL; /* adding 30.06.05 */
+  ppaxesmdl->size_of_user_data = 0;
   ppaxesmdl->relationship.psons = (sciSons *) NULL;
   ppaxesmdl->relationship.plastsons = (sciSons *) NULL;
   ppaxesmdl->callback = (char *)NULL;
@@ -215,14 +227,14 @@ int C2F(graphicsmodels) ()
   
   /* F.Leray 09.04.04 */
   if (sciInitFontContext (paxesmdl) == -1)
-    {
-      sciDelThisToItsParent (paxesmdl, sciGetParent (paxesmdl));
-      sciDelHandle (paxesmdl);
-      FREE(paxesmdl->pfeatures);	  
-      FREE(paxesmdl);
-      strcpy(error_message,"Default axes cannot be create");
-      return 0;
-    }
+  {
+    sciDelThisToItsParent (paxesmdl, sciGetParent (paxesmdl));
+    sciDelHandle (paxesmdl);
+    FREE(paxesmdl->pfeatures);	  
+    FREE(paxesmdl);
+    strcpy(error_message,"Default axes cannot be create");
+    return 0;
+  }
   
   ppaxesmdl->logflags[0] = 'n';
   ppaxesmdl->logflags[1] = 'n';
@@ -257,7 +269,7 @@ int C2F(graphicsmodels) ()
   dir= 'l'; ppaxesmdl->axes.ydir=dir;      
 
   /* BU HERE */
-/*   ppaxesmdl = ppaxesmdl; */
+  /*   ppaxesmdl = ppaxesmdl; */
   for (i=0 ; i<4 ; i++){  
     ppaxesmdl->axes.xlim[i]=  Cscale.xtics[i];
     ppaxesmdl->axes.ylim[i]=  Cscale.ytics[i]; }
@@ -343,13 +355,16 @@ int C2F(graphicsmodels) ()
   ppaxesmdl->clip_region[1] = 0.;
   ppaxesmdl->clip_region[2] = 0.;
   ppaxesmdl->clip_region[3] = 0.;
+
+  /* the model has not been changed !!! */
+  ppaxesmdl->clip_region_set = 0 ;
   
   ppaxesmdl->tight_limits = FALSE;
   
   ppaxesmdl->isselected = FALSE;
 
   ppaxesmdl->visible = sciGetVisibility(pfiguremdl);
-/*   ppaxesmdl->drawlater = FALSE; */
+  /*   ppaxesmdl->drawlater = FALSE; */
   ppaxesmdl->isclip = -1;
   
   ppaxesmdl->pPopMenu = (sciPointObj *)NULL;
@@ -367,29 +382,32 @@ int C2F(graphicsmodels) ()
     return -1;
   sciSetEntityType (ppobj->mon_title, SCI_LABEL);
   if (((ppobj->mon_title)->pfeatures = MALLOC ((sizeof (sciLabel)))) == NULL)
-    {
-      FREE((ppobj->mon_title));
-      return -1;
-    }
+  {
+    FREE((ppobj->mon_title));
+    return -1;
+  }
   
   
   if (sciAddNewHandle ((ppobj->mon_title)) == -1)
-    {
-      FREE(pLABEL_FEATURE((ppobj->mon_title)));
+  {
+    FREE(pLABEL_FEATURE((ppobj->mon_title)));
       FREE((ppobj->mon_title));
       return -1;
-    }
+  }
   
   if (!(sciAddThisToItsParent ((ppobj->mon_title), paxesmdl)))
-    {
-      sciDelHandle ((ppobj->mon_title));
-      FREE(pLABEL_FEATURE((ppobj->mon_title)));
-      FREE((ppobj->mon_title));
-      return -1;
-    }
+  {
+    sciDelHandle ((ppobj->mon_title));
+    FREE(pLABEL_FEATURE((ppobj->mon_title)));
+    FREE((ppobj->mon_title));
+    return -1;
+  }
       
   sciSetCurrentSon ((ppobj->mon_title), (sciPointObj *) NULL);
-      
+  
+  pLABEL_FEATURE ((ppobj->mon_title))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_title))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_title))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_title))->text.relationship.plastsons = (sciSons *) NULL;
 
@@ -397,8 +415,12 @@ int C2F(graphicsmodels) ()
   pLABEL_FEATURE ((ppobj->mon_title))->text.callbacklen = 0;
   /*   pLABEL_FEATURE ((ppobj->mon_title))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_title))); */
   pLABEL_FEATURE ((ppobj->mon_title))->visible = sciGetVisibility(paxesmdl);
+  pLABEL_FEATURE (ppobj->mon_title)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_title)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_title)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_title)->position[1] = 0;
   
-  if ((pLABEL_FEATURE ((ppobj->mon_title))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_title))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_title), sciGetParent ((ppobj->mon_title)));
@@ -457,17 +479,23 @@ int C2F(graphicsmodels) ()
     }
       
   sciSetCurrentSon ((ppobj->mon_x_label), (sciPointObj *) NULL);
-      
+  
+  pLABEL_FEATURE ((ppobj->mon_x_label))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_x_label))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.callbacklen = 0;
- /*  pLABEL_FEATURE ((ppobj->mon_x_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_x_label))); */
+  /*  pLABEL_FEATURE ((ppobj->mon_x_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_x_label))); */
   pLABEL_FEATURE ((ppobj->mon_x_label))->visible = sciGetVisibility(paxesmdl);
-     
+  pLABEL_FEATURE (ppobj->mon_x_label)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_x_label)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_x_label)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_x_label)->position[1] = 0;
 
-  if ((pLABEL_FEATURE ((ppobj->mon_x_label))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_x_label))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_x_label), sciGetParent ((ppobj->mon_x_label)));
@@ -526,17 +554,23 @@ int C2F(graphicsmodels) ()
     }
       
   sciSetCurrentSon ((ppobj->mon_y_label), (sciPointObj *) NULL);
-      
+
+  pLABEL_FEATURE ((ppobj->mon_y_label))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_y_label))->size_of_user_data = 0;
+ 
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.callbacklen = 0;
- /*  pLABEL_FEATURE ((ppobj->mon_y_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_y_label))); */
+  /*  pLABEL_FEATURE ((ppobj->mon_y_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_y_label))); */
   pLABEL_FEATURE ((ppobj->mon_y_label))->visible = sciGetVisibility(paxesmdl);
-  
+  pLABEL_FEATURE (ppobj->mon_y_label)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_y_label)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_y_label)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_y_label)->position[1] = 0;
 
-  if ((pLABEL_FEATURE ((ppobj->mon_y_label))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_y_label))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_y_label), sciGetParent ((ppobj->mon_y_label)));
@@ -594,17 +628,23 @@ int C2F(graphicsmodels) ()
     }
       
   sciSetCurrentSon ((ppobj->mon_z_label), (sciPointObj *) NULL);
-      
+
+  pLABEL_FEATURE ((ppobj->mon_z_label))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_z_label))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.callbacklen = 0;
- /*  pLABEL_FEATURE ((ppobj->mon_z_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_z_label))); */
+  /*  pLABEL_FEATURE ((ppobj->mon_z_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_z_label))); */
   pLABEL_FEATURE ((ppobj->mon_z_label))->visible = sciGetVisibility(paxesmdl);
+  pLABEL_FEATURE (ppobj->mon_z_label)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_z_label)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_z_label)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_z_label)->position[1] = 0;
   
-  
-  if ((pLABEL_FEATURE ((ppobj->mon_z_label))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_z_label))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_z_label), sciGetParent ((ppobj->mon_z_label)));
@@ -652,9 +692,9 @@ sciInitGraphicContext (sciPointObj * pobj)
 
   /* F.Leray debug 30.03.04*/
   /*
-  sciGraphicContext * psciGC_pobj = sciGetGraphicContext(pobj); 
-  sciGraphicContext * psciGC_paxe = sciGetGraphicContext(paxesmdl);
-  sciGraphicContext * psciGC_pfig = sciGetGraphicContext(pfiguremdl);
+    sciGraphicContext * psciGC_pobj = sciGetGraphicContext(pobj); 
+    sciGraphicContext * psciGC_paxe = sciGetGraphicContext(paxesmdl);
+    sciGraphicContext * psciGC_pfig = sciGetGraphicContext(pfiguremdl);
   */
   switch (sciGetEntityType (pobj))
     {
@@ -742,6 +782,7 @@ sciInitGraphicContext (sciPointObj * pobj)
     case SCI_MENUCONTEXT:
     case SCI_STATUSB: 
     case SCI_LEGEND: /* Adding a graphic context to legend object F.Leray 21.01.05 */
+    case SCI_TEXT:
       (sciGetGraphicContext(pobj))->backgroundcolor =	sciGetBackground (sciGetParent (pobj)) - 1;
       (sciGetGraphicContext(pobj))->foregroundcolor =	sciGetForeground (sciGetParent (pobj)) - 1;
       (sciGetGraphicContext(pobj))->fillstyle =	sciGetFillStyle (sciGetParent (pobj));
@@ -758,12 +799,12 @@ sciInitGraphicContext (sciPointObj * pobj)
       return 0;
       break;
     case SCI_AGREG:
-    case SCI_TEXT:
     case SCI_TITLE:
     case SCI_PANNER:		/* pas de context graphics */
     case SCI_SBH:		/* pas de context graphics */
     case SCI_SBV:		/* pas de context graphics */
     case SCI_LABEL: /* F.Leray 28.05.04 */
+    case SCI_UIMENU:
     default:
       return -1;
       break;
@@ -780,7 +821,7 @@ sciInitGraphicContext (sciPointObj * pobj)
 int
 sciInitFontContext (sciPointObj * pobj)
 {
-   /*   int aa[5]; */ /* for debug purpose only F.Leray 13.01.05 */
+  /*   int aa[5]; */ /* for debug purpose only F.Leray 13.01.05 */
 
   /* 
    * initialisation du contexte font par defaut 
@@ -813,7 +854,7 @@ sciInitFontContext (sciPointObj * pobj)
       (sciGetFontContext(pobj))->textorientation/*  = aa[4] */ =  (sciGetFontContext(sciGetParent(pobj)))->textorientation;
       (sciGetFontContext(pobj))->fontnamelen    /*  = aa[5] */ =  (sciGetFontContext(sciGetParent(pobj)))->fontnamelen; 
       if (((sciGetFontContext(pobj))->pfontname =
-	   calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	   CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 		   sizeof (char))) == NULL)
 	{
 	  sciprint ("No more Memory for fontname\n"); 
@@ -830,9 +871,10 @@ sciInitFontContext (sciPointObj * pobj)
 	  (sciGetFontContext(pobj))->fontdeciwidth = 100;
 	  (sciGetFontContext(pobj))->textorientation = 0;
 	  (sciGetFontContext(pobj))->fontnamelen=1; /*fontname not used */
+	  (sciGetFontContext(pobj))->textorientation = 0;
 
 	  if (((sciGetFontContext(pobj))->pfontname =
-	       calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	       CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 		       sizeof (char))) == NULL)
 	    {
 	      sciprint ("No more Memory for fontname\n"); 
@@ -856,11 +898,11 @@ sciInitFontContext (sciPointObj * pobj)
 	  (sciGetFontContext(pobj))->foregroundcolor/*  = aa[1] */ = (sciGetFontContext(plabelmdl))->foregroundcolor;
 	  (sciGetFontContext(pobj))->fonttype  /*=        aa[2] */ = (sciGetFontContext(plabelmdl))->fonttype;
  	  (sciGetFontContext(pobj))->fontdeciwidth  /*=   aa[3] */ = (sciGetFontContext(plabelmdl))->fontdeciwidth;
-	  (sciGetFontContext(pobj))->textorientation/*  = aa[4] */ = (sciGetFontContext(plabelmdl))->textorientation;
 	  (sciGetFontContext(pobj))->fontnamelen  /*=     aa[5] */ = (sciGetFontContext(plabelmdl))->fontnamelen; /*fontname not used */
+	  (sciGetFontContext(pobj))->textorientation/*  = aa[4] */ = (sciGetFontContext(plabelmdl))->textorientation;
 	  
 	  if (
-	      ((sciGetFontContext(pobj))->pfontname = calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	      ((sciGetFontContext(pobj))->pfontname = CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 							      sizeof (char))) == NULL)
 	    {
 	      sciprint ("No more Memory for fontname\n");
@@ -879,7 +921,7 @@ sciInitFontContext (sciPointObj * pobj)
 	  (sciGetFontContext(pobj))->fontnamelen=1; /*fontname not used */
 	  
 	  if (
-	      ((sciGetFontContext(pobj))->pfontname = calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	      ((sciGetFontContext(pobj))->pfontname = CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 							      sizeof (char))) == NULL)
 	    {
 	      sciprint ("No more Memory for fontname\n");
@@ -896,7 +938,7 @@ sciInitFontContext (sciPointObj * pobj)
 	  (sciGetFontContext(pobj))->fontnamelen /* =     aa[5] */ =(sciGetFontContext(paxesmdl))->fontnamelen; /*fontname not used */
 	  
 	  if (
-	      ((sciGetFontContext(pobj))->pfontname = calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	      ((sciGetFontContext(pobj))->pfontname = CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 							      sizeof (char))) == NULL)
 	    {
 	      sciprint ("No more Memory for fontname\n");
@@ -916,7 +958,7 @@ sciInitFontContext (sciPointObj * pobj)
 	  (sciGetFontContext(pobj))->fontnamelen=1; /*fontname not used */
 	  
 	  if (
-	      ((sciGetFontContext(pobj))->pfontname = calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	      ((sciGetFontContext(pobj))->pfontname = CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 							      sizeof (char))) == NULL)
 	    {
 	      sciprint ("No more Memory for fontname\n");
@@ -936,7 +978,7 @@ sciInitFontContext (sciPointObj * pobj)
 	  (sciGetFontContext(pobj))->fontnamelen =     (sciGetFontContext(pfiguremdl))->fontnamelen; /*fontname not used */
 	  
 	  if (
-	      ((sciGetFontContext(pobj))->pfontname = calloc ((sciGetFontContext(pobj))->fontnamelen + 1,
+	      ((sciGetFontContext(pobj))->pfontname = CALLOC ((sciGetFontContext(pobj))->fontnamelen + 1,
 							      sizeof (char))) == NULL)
 	    {
 	      sciprint ("No more Memory for fontname\n");
@@ -957,6 +999,7 @@ sciInitFontContext (sciPointObj * pobj)
     case SCI_SURFACE:
     case SCI_LIGHT:
     case SCI_AGREG:
+    case SCI_UIMENU:
     default:
       return -1;
       break;
@@ -973,6 +1016,9 @@ void initsubwin()  /* Interesting / F.Leray 05.04.04 */
   Cscale2default(); 
   psubwin=sciGetSelectedSubWin (sciGetCurrentFigure ());
   
+  pSUBWIN_FEATURE (psubwin)->user_data = (int *) NULL; /* adding 30.06.05 */
+  pSUBWIN_FEATURE (psubwin)->size_of_user_data = 0;
+  
   pSUBWIN_FEATURE (psubwin)->WRect[0]   = 0;
   pSUBWIN_FEATURE (psubwin)->WRect[1]   = 0;
   pSUBWIN_FEATURE (psubwin)->WRect[2]   = 1;
@@ -1008,7 +1054,7 @@ void initsubwin()  /* Interesting / F.Leray 05.04.04 */
   pSUBWIN_FEATURE (psubwin)->axes.subint[2] =  1;
   pSUBWIN_FEATURE (psubwin)->axes.limits[0]  = 0;  
   pSUBWIN_FEATURE (psubwin)->visible = TRUE;
-/*   pSUBWIN_FEATURE (psubwin)->drawlater = FALSE; */
+  /*   pSUBWIN_FEATURE (psubwin)->drawlater = FALSE; */
   pSUBWIN_FEATURE (psubwin)->is3d = FALSE;  
   pSUBWIN_FEATURE (psubwin)->alpha  = 0.0;
   pSUBWIN_FEATURE (psubwin)->theta  = 270.0;
@@ -1049,6 +1095,9 @@ int InitFigureModel()
   pFIGURE_FEATURE (pfiguremdl)->visible = TRUE;
   pFIGURE_FEATURE (pfiguremdl)->auto_redraw = TRUE; /* by default, we draw live */ /* F.Leray 29.12.04 modified on 04.01.05 */
 
+  pFIGURE_FEATURE (pfiguremdl)->user_data = (int *) NULL; /* pour completude */
+  pFIGURE_FEATURE (pfiguremdl)->size_of_user_data = 0; /* pour completude */
+
   pFIGURE_FEATURE (pfiguremdl)->numsubwinselected = 0; 
   pFIGURE_FEATURE (pfiguremdl)->pixmap = 0; 
   pFIGURE_FEATURE (pfiguremdl)->wshow = 0;
@@ -1085,6 +1134,9 @@ int InitAxesModel()
   ppaxesmdl->axes.ydir='l';  
   ppaxesmdl->axes.rect  = 0;
   
+  ppaxesmdl->user_data = (int *) NULL; /* to be complete */
+  ppaxesmdl->size_of_user_data = 0;
+
   for (i=0 ; i<7 ; i++)
     ppaxesmdl->axes.limits[i]  = 0;
 
@@ -1167,8 +1219,8 @@ int InitAxesModel()
   ppaxesmdl->FRect[5]   = 1.0;   /* zmax */
   ppaxesmdl->isselected = FALSE;
   ppaxesmdl->visible = sciGetVisibility(pfiguremdl); 
-/* /\*   ppaxesmdl->drawlater = sciGetDrawLater(pfiguremdl); *\/ */
-/*   ppaxesmdl->drawlater = FALSE; */
+  /* /\*   ppaxesmdl->drawlater = sciGetDrawLater(pfiguremdl); *\/ */
+  /*   ppaxesmdl->drawlater = FALSE; */
 
   ppaxesmdl->isclip = -1;  
   ppaxesmdl->pPopMenu = (sciPointObj *)NULL;
@@ -1219,16 +1271,23 @@ int InitAxesModel()
     }
       
   sciSetCurrentSon ((ppobj->mon_title), (sciPointObj *) NULL);
-      
+  
+  pLABEL_FEATURE ((ppobj->mon_title))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_title))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_title))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_title))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_title))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_title))->text.callbacklen = 0; 
- /*  pLABEL_FEATURE ((ppobj->mon_title))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_title)));  */
+  /*  pLABEL_FEATURE ((ppobj->mon_title))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_title)));  */
   pLABEL_FEATURE ((ppobj->mon_title))->visible = sciGetVisibility(paxesmdl);
+  pLABEL_FEATURE (ppobj->mon_title)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_title)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_title)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_title)->position[1] = 0;
 
-  if ((pLABEL_FEATURE ((ppobj->mon_title))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_title))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_title), sciGetParent ((ppobj->mon_title)));
@@ -1287,17 +1346,23 @@ int InitAxesModel()
     }
       
   sciSetCurrentSon ((ppobj->mon_x_label), (sciPointObj *) NULL);
-      
+  
+  pLABEL_FEATURE ((ppobj->mon_x_label))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_x_label))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_x_label))->text.callbacklen = 0; 
- /*  pLABEL_FEATURE ((ppobj->mon_x_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_x_label)));  */
+  /*  pLABEL_FEATURE ((ppobj->mon_x_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_x_label)));  */
   pLABEL_FEATURE ((ppobj->mon_x_label))->visible = sciGetVisibility(paxesmdl);
-      
+  pLABEL_FEATURE (ppobj->mon_x_label)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_x_label)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_x_label)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_x_label)->position[1] = 0;
 
-  if ((pLABEL_FEATURE ((ppobj->mon_x_label))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_x_label))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_x_label), sciGetParent ((ppobj->mon_x_label)));
@@ -1356,17 +1421,23 @@ int InitAxesModel()
     }
       
   sciSetCurrentSon ((ppobj->mon_y_label), (sciPointObj *) NULL);
-      
+  
+  pLABEL_FEATURE ((ppobj->mon_y_label))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_y_label))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_y_label))->text.callbacklen = 0; 
- /*  pLABEL_FEATURE ((ppobj->mon_y_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_y_label)));  */
+  /*  pLABEL_FEATURE ((ppobj->mon_y_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_y_label)));  */
   pLABEL_FEATURE ((ppobj->mon_y_label))->visible = sciGetVisibility(paxesmdl);
-    
-
-  if ((pLABEL_FEATURE ((ppobj->mon_y_label))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  pLABEL_FEATURE (ppobj->mon_y_label)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_y_label)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_y_label)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_y_label)->position[1] = 0;
+  
+  if ((pLABEL_FEATURE ((ppobj->mon_y_label))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_y_label), sciGetParent ((ppobj->mon_y_label)));
@@ -1422,19 +1493,25 @@ int InitAxesModel()
       FREE((ppobj->mon_z_label));
       return -1;
     }
-      
+  
   sciSetCurrentSon ((ppobj->mon_z_label), (sciPointObj *) NULL);
-      
+  
+  pLABEL_FEATURE ((ppobj->mon_z_label))->user_data = (int *) NULL;
+  pLABEL_FEATURE ((ppobj->mon_z_label))->size_of_user_data = 0;
+  
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.relationship.psons = (sciSons *) NULL;
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.relationship.plastsons = (sciSons *) NULL;
 
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.callback = (char *)NULL;
   pLABEL_FEATURE ((ppobj->mon_z_label))->text.callbacklen = 0; 
- /*  pLABEL_FEATURE ((ppobj->mon_z_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_z_label)));  */
+  /*  pLABEL_FEATURE ((ppobj->mon_z_label))->visible = sciGetVisibility(sciGetParentFigure((ppobj->mon_z_label)));  */
   pLABEL_FEATURE ((ppobj->mon_z_label))->visible = sciGetVisibility(paxesmdl);
-      
+  pLABEL_FEATURE (ppobj->mon_z_label)->auto_position = TRUE;
+  pLABEL_FEATURE (ppobj->mon_z_label)->auto_rotation = TRUE;
+  pLABEL_FEATURE (ppobj->mon_z_label)->position[0] = 0;
+  pLABEL_FEATURE (ppobj->mon_z_label)->position[1] = 0;
 
-  if ((pLABEL_FEATURE ((ppobj->mon_z_label))->text.ptextstring =calloc (1, sizeof (char))) == NULL)
+  if ((pLABEL_FEATURE ((ppobj->mon_z_label))->text.ptextstring =CALLOC (1, sizeof (char))) == NULL)
     {
       sciprint("No more place to allocates text string, try a shorter string");
       sciDelThisToItsParent ((ppobj->mon_z_label), sciGetParent ((ppobj->mon_z_label)));
@@ -1490,7 +1567,7 @@ int ResetFigureToDefaultValues(sciPointObj * pobj)
   m = pFIGURE_FEATURE (pfiguremdl)->numcolors;
   /* try to install the colormap in the graphic context */
   C2F(dr)("xset","colormap",&m,&n,&succeed,PI0,PI0,PI0,
-	 pFIGURE_FEATURE(pfiguremdl)->pcolormap,PD0,PD0,PD0,0L,0L);
+	  pFIGURE_FEATURE(pfiguremdl)->pcolormap,PD0,PD0,PD0,0L,0L);
 
   if(succeed == 1){ /* failed to allocate or xinit (for Gif driver) was missing */
     sciprint ("Failed to change colormap : Allocation failed or missing xinit detected\n");
@@ -1643,6 +1720,7 @@ sciInitGraphicMode (sciPointObj * pobj)
     case SCI_STATUSB:
     case SCI_AGREG:
     case SCI_LABEL: /* F.Leray 28.05.04 */
+    case SCI_UIMENU:
     default:
       sciprint ("This object haven't any graphic mode\n");
       return -1;
