@@ -56,6 +56,7 @@
 #define SB_HORZ 0
 #endif 
 #define NUMCOLORS_SCI 32
+#define SCI_FIGURE_NAME_LENGTH 256
 /* Renvoi le pointeur sur la structure */
 
 #define pFIGURE_FEATURE(pointobj)      ((sciFigure        *)pointobj->pfeatures)/** */
@@ -85,6 +86,8 @@
 
 #define pLABEL_FEATURE(pointobj)       ((sciLabel         *)pointobj->pfeatures)/** */
 
+#define pUIMENU_FEATURE(pointobj)       ((sciUimenu       *)pointobj->pfeatures)/** */
+
 #ifndef WIN32
 typedef unsigned short HMENU;
 typedef void *HFONT;                                         
@@ -134,59 +137,74 @@ POINT3D;
  * @memo SCI_STATUSB,
  */
 typedef enum
-{/**Entity type FIGURE*/
-  SCI_FIGURE,		
-  /**Entity type SUBWINDOW*/
-  SCI_SUBWIN,		
-  /**Entity type TEXT*/
-  SCI_TEXT,			
-  /**Entity type TITLE */
-  SCI_TITLE,		
-  /**Entity type LEGEND */
-  SCI_LEGEND,		
-  /**Entity type ARC */
-  SCI_ARC,			
-  /**Entity type POLYLINE*/
-  SCI_POLYLINE,	    
-  /**Entity type RECTANGLE*/
-  SCI_RECTANGLE,    
-  /**Entity type SURFACE*/
-  SCI_SURFACE,
-  /**Entity type MERGE*/  /* DJ.A 30/12 */
-  SCI_MERGE,
-  /**Entity type LIGHT*/
-  SCI_LIGHT,		
-  /**Entity type AXIS*/
-  SCI_AXIS,	
-  /**Entity type AXES*/
-  SCI_AXES,	  
-  /**Entity type SEGS*/
-  SCI_SEGS,	
-  /**Entity type GRAYPLOT*/
-  SCI_GRAYPLOT, 
-  /**Entity type FEC*/
-  SCI_FEC,				
-  /**Entity type PANNER*/
-  SCI_PANNER,		
-  /**Entity type HORIZONTALL SCROLL BAR */
-  SCI_SBH,			
-  /**Entity type VERTICALL SCROLL BAR*/
-  SCI_SBV,			
-  /**Entity type MENU*/
-  SCI_MENU,			
-  /**Entity type CONTEXT MENU*/
-  SCI_MENUCONTEXT,	
-  /**Entity type STATUS BAR*/
-  SCI_STATUSB,	    
-  /**Entity type AGREGATION */
-  SCI_AGREG,			
-  /**Entity type LABEL created by F.Leray 26.05.04 */
-  SCI_LABEL
-}
+  {/**Entity type FIGURE*/
+    SCI_FIGURE,		
+    /**Entity type SUBWINDOW*/
+    SCI_SUBWIN,		
+    /**Entity type TEXT*/
+    SCI_TEXT,			
+    /**Entity type TITLE */
+    SCI_TITLE,		
+    /**Entity type LEGEND */
+    SCI_LEGEND,		
+    /**Entity type ARC */
+    SCI_ARC,			
+    /**Entity type POLYLINE*/
+    SCI_POLYLINE,	    
+    /**Entity type RECTANGLE*/
+    SCI_RECTANGLE,    
+    /**Entity type SURFACE*/
+    SCI_SURFACE,
+    /**Entity type MERGE*/  /* DJ.A 30/12 */
+    SCI_MERGE,
+    /**Entity type LIGHT*/
+    SCI_LIGHT,		
+    /**Entity type AXIS*/
+    SCI_AXIS,	
+    /**Entity type AXES*/
+    SCI_AXES,	  
+    /**Entity type SEGS*/
+    SCI_SEGS,	
+    /**Entity type GRAYPLOT*/
+    SCI_GRAYPLOT, 
+    /**Entity type FEC*/
+    SCI_FEC,				
+    /**Entity type PANNER*/
+    SCI_PANNER,		
+    /**Entity type HORIZONTALL SCROLL BAR */
+    SCI_SBH,			
+    /**Entity type VERTICALL SCROLL BAR*/
+    SCI_SBV,			
+    /**Entity type MENU*/
+    SCI_MENU,			
+    /**Entity type CONTEXT MENU*/
+    SCI_MENUCONTEXT,	
+    /**Entity type STATUS BAR*/
+    SCI_STATUSB,	    
+    /**Entity type Compound */
+    SCI_AGREG,			
+    /**Entity type LABEL created by F.Leray 26.05.04 */
+    SCI_LABEL,
+    /**Entity type UIMENU created by A.C 28.09.05 **/
+    SCI_UIMENU
+  }
 /**Struct of Entity type*/
 sciEntityType;	
 
 
+typedef struct _Vertices 
+{
+  int value_xm;
+  int value_ym;
+  double value_x;
+  double value_y;
+  double value_z;
+  
+  struct _Vertices * pNext;
+} 
+Vertices;
+
+
 /**@name sciPointObj
  * Used to determine the feature and the type of the entity
  */
@@ -228,12 +246,12 @@ sciHandleTab;
 
 /**@name Sons
  * Used to determine the hierarchy
-  sciPointObj *pointobj;
-  sciSons     *pnext;
- */
+ sciPointObj *pointobj;
+ sciSons     *pnext;
+*/
 typedef struct tagSons
 {
-	/** */
+  /** */
   struct tagSons *pprev;	  
   /**  is the pointer of next the son's structure      */
   sciPointObj *pointobj;	
@@ -327,8 +345,8 @@ sciGraphicContext;
   ----------------------*/
 
 /* Changing those lines, be carreful that
- the new attribut is update in the sciset and sciget routines
- */
+   the new attribut is update in the sciset and sciget routines
+*/
 /** */
 #define SCI_DONT_CARE          0
 /** */
@@ -421,7 +439,7 @@ typedef struct
   sciGraphicContext graphiccontext; /* the only property used here is background */
   sciPointObj * originalsubwin0011;
   /** specifies the title for this window  */
-  char name[80];	 
+  char name[SCI_FIGURE_NAME_LENGTH];	 
   /** specifies le length of the string name */
   int namelen;
   /** specifies the number of this window            */
@@ -454,9 +472,11 @@ typedef struct
   int wshow ;
   
   BOOL allredraw;
-/*   BOOL force_draw; /\* exclusively used with draw command to force the drawing of the object *\/ */
+  /*   BOOL force_draw; /\* exclusively used with draw command to force the drawing of the object *\/ */
 
   sciFont fontcontext; /* F.Leray 08.04.04 */
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
   
 }/** */
 sciFigure;  
@@ -471,6 +491,7 @@ typedef struct
   sciRelationShip relationship;
   /** */
   sciFont fontcontext;
+  sciGraphicContext graphiccontext; /* the only properties used by Text are foreground and background */
   /** */
   char *ptextstring;
   /** position in scilab window (not pixel) */
@@ -479,7 +500,10 @@ typedef struct
   double y;
   /***/
   double wh[2];
-  BOOL fill;
+  BOOL fill; /* to distinguish between xstring and xstringb */
+  BOOL isboxed  ;
+  BOOL isline   ; /* switch the contour of the box */
+  BOOL isfilled ; /* switch the transparency of the box */
   /** */
   double z; /**DJ.Abdemouche 2003**/
   unsigned int textlen;
@@ -490,10 +514,14 @@ typedef struct
   int callbackevent; 
   /** specifies if this object is visble             */
   BOOL visible;
-  int isclip;
-  double clip_region[4];
-  int clip_region_set;
-}/** */
+  int isclip; /* this is the scilab clip_state propety */
+  double clip_region[4]; /* scilab clip_box property */
+  int clip_region_set; /* false if the clip state of the object has not already been set */
+  
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
+
+}
 sciText;  
 
 
@@ -503,22 +531,22 @@ sciText;
  * Enumeration used to specify the title place 
  */
 typedef enum
-{
-  /** */
-  SCI_LEGEND_OUTSIDE = -1,
-  /** */
-  SCI_LEGEND_IN_INSIDE = 0,
-  /** */
-  SCI_LEGEND_IN_UPPER_LEFT = 1,
-  /** */
-  SCI_LEGEND_IN_UPPER_RIGHT = 2,
-  /** */
-  SCI_LEGEND_IN_LOWER_LEFT = 3,
-  /** */
-  SCI_LEGEND_IN_LOWER_RIGHT = 4,
-  /** */
-  SCI_LEGEND_IN_SPECIFIED = 5
-}  /** */
+  {
+    /** */
+    SCI_LEGEND_OUTSIDE = -1,
+    /** */
+    SCI_LEGEND_IN_INSIDE = 0,
+    /** */
+    SCI_LEGEND_IN_UPPER_LEFT = 1,
+    /** */
+    SCI_LEGEND_IN_UPPER_RIGHT = 2,
+    /** */
+    SCI_LEGEND_IN_LOWER_LEFT = 3,
+    /** */
+    SCI_LEGEND_IN_LOWER_RIGHT = 4,
+    /** */
+    SCI_LEGEND_IN_SPECIFIED = 5
+  }  /** */
 sciLegendPlace;
 
 
@@ -541,7 +569,7 @@ typedef struct
   int width;		     
   /** specifies the frame's height */
   int height;		     
-   /** */
+  /** */
   BOOL isselected;
   /** specifies if the legend is surrounded */
   BOOL issurround;	     
@@ -557,6 +585,8 @@ typedef struct
   int isclip;
   double clip_region[4];
   int clip_region_set;
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }/** */
 sciLegend;  
 
@@ -565,12 +595,12 @@ sciLegend;
  * Enumeration used to specify the title place 
  */
 typedef enum
-{
-  /** */
-  SCI_TITLE_IN_TOP = 0,
-  /** */
-  SCI_TITLE_IN_BOTTOM = 1,
-}/** */
+  {
+    /** */
+    SCI_TITLE_IN_TOP = 0,
+    /** */
+    SCI_TITLE_IN_BOTTOM = 1,
+  }/** */
 sciTitlePlace;	
 
 
@@ -614,9 +644,40 @@ typedef struct
   int isclip;
   double clip_region[4];
   int clip_region_set;
-}/** */
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
+  BOOL isfilled; /* to know if a label has a colored frame as background */
+  BOOL auto_position; /* automatic or manual position selection for label */
+  BOOL auto_rotation; /* automatic or manual rotation selection for label (depends on the current view mode 2d or 3d mainly for y and z labels) */
+                      /* not implemented for titles */
+  double position[2]; /* the (x,y) coord. of the label's position */
+
+}
 sciLabel;  
 
+typedef struct
+{
+  sciRelationShip relationship;
+  /** */
+  sciText label; /* Callback in sciText struct. */
+  /** specifies if this object is visble             */
+  BOOL visible;
+  BOOL handle_visible;
+  BOOL Enable;
+
+  int MenuPosition;
+  int CallbackType;
+
+  #if WIN32
+	HMENU hMenu;
+	int IDM_this;
+  #endif
+
+
+}/** */
+sciUimenu;
+
+
 
 typedef struct 
 {  
@@ -651,7 +712,7 @@ typedef struct
   double  limits[7]; /* = 1 set tight limits = 0 set axes auto shape */
   integer flag[3]; /* 3d options */
   
-   /* F.Leray 07.10.04 REMOVE AAINT*/
+  /* F.Leray 07.10.04 REMOVE AAINT*/
   int nbsubtics[3]; /* Don't touch to subint because also deals with drawaxis: AXES structure has multiple uses... */ /* F.Leray 07.10.04 */
 
   BOOL reverse[3]; /* if TRUE, it means that the corresponding axe is reversed */
@@ -675,7 +736,7 @@ typedef struct
   /** */
   /** specifies the title for this window  */
   char name[sizeof ("ScilabGraphic") + 4];	    
-    /** */
+  /** */
   int namelen;
   /** specifies the number for this window                   */
   int number;			
@@ -701,7 +762,7 @@ typedef struct
   double ARect[4]; /* margins*/
   int zoomy_kp;
   /* ZRect_kp is now useless : when unzooming we deal with SRect values */
-  double ZRect[4]; /* reversed for zoom only to avoid using FRect as zoom box AND computed box */ /* F.Leray 09.12.04 */
+  double ZRect[6]; /* reversed for zoom only to avoid using FRect as zoom box AND computed box */ /* F.Leray 09.12.04 */
 
   char logflags[3]; /* Z has a logflag now F.Leray 03.11.04 */
   int grid[3];
@@ -745,6 +806,18 @@ typedef struct
   /*  An internal state used as subtics flag (auto == FALSE, TRUE == manual) named flagNax */
   BOOL flagNax;
   int surfcounter; /* used to merge 3d objects */
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
+  
+  int YGradMostOnLeft;
+  int YGradMostOnRight;
+  int XGradMostOnTop;
+  int XGradMostOnBottom;
+
+  BOOL firsttime_x;
+  BOOL firsttime_y;
+
+  Vertices * vertices_list; /* F.Leray 30.08.05 : stores the (x,y) coord. in term of user data coord. + pixel value on screen */
 }/** */
 sciSubWindow;  
 
@@ -775,7 +848,7 @@ typedef struct
   double alphabegin;		
   /** end at alpha2   */
   double alphaend;
-  int fill;		
+  int fill; /* another flag to specify wether we should have a box around the text */
   /** */
   BOOL isselected;
   /** specifies the text scilab code for the callback associated with this entity */
@@ -789,6 +862,10 @@ typedef struct
   double clip_region[4];
   int clip_region_set;
   double z; /**DJ.Abdemouche 2003**/
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
+  BOOL isfilled; /* to know if an arc is filled or not */
+  /* the isline property is inside the associated graphiccontext */
 }  /** */
 sciArc;
 
@@ -820,12 +897,11 @@ typedef struct
 {
   sciRelationShip relationship;
   sciGraphicContext graphiccontext;
-  POINT2D *pvector;		/* vecteur de points redondant, for future developpement*/
   double *pvx;			/* vecteur des points x doublon avec pvector je les garde pour compatiblite*/
   double *pvy;			/* vecteur des points y doublon avec pvector*/
   double *pvz; /**DJ.Abdemouche 2003**/
-  int n1;                       /** number of point */
-  int n2;                       /** numbre of curve if Plot **/ 
+  int n1;                       /** number of points */
+  int n2;                       /** numbre of curves if Plot **/ 
   int closed;                   /** is it a closed polyline */
   int plot;                     /** defines the polyline_style (interpolated, staircase, bar_plot,...) : is it simple poly or a plot (Plot2d /Plot2d1/.../Plot2d4) */
   BOOL isselected;
@@ -837,6 +913,21 @@ typedef struct
   int isclip;
   double clip_region[4];
   int clip_region_set;
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
+  BOOL isfilled; /* to know if a polyline is filled or not */
+  BOOL isinterpshaded;
+  int *scvector;
+  int dim_icv; /* dimension of the interpolated color vector (3 or 4) */
+  int bar_layout;
+
+  double * x_shift;
+  double * y_shift;
+  double * z_shift;
+  
+  double bar_width;
+  
+  double arsize_factor; /* to be able to enlarge arrow size without changing the line thickness */
 }
 sciPolyline;  /** */
 
@@ -853,7 +944,6 @@ typedef struct
   double width;
   double height;
   int fillflag;
-  int fillcolor;
   int str; 
   integer strwidth;
   integer strheight;
@@ -871,18 +961,21 @@ typedef struct
   double z;  /** rectangle */
 
   BOOL flagstring; /* flag used to determine wether the rectangle is used to surround a string : used when axes is reversed */
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
+  BOOL isfilled; /* to know if a rectangle is filled or not */
 }
 sciRectangle;  /** */
 
 
 typedef enum
-{
-	SCI_FAC3D,
-	SCI_CONTOUR,
-	SCI_PARAM3D,
-	SCI_PARAM3D1,
-	SCI_PLOT3D,
-}
+  {
+    SCI_FAC3D,
+    SCI_CONTOUR,
+    SCI_PARAM3D,
+    SCI_PARAM3D1,
+    SCI_PLOT3D,
+  }
 sciTypeOf3D;
 
 
@@ -922,12 +1015,11 @@ typedef struct
   int m3,n3;
   int m3n,n3n;
 
-  POINT2D *pproj;			/* projections on 2d */
   integer flag[3];
   double ebox[6];
   int flagcolor; /* this flag indicates the type of the color of how the facet have to be colored 
-                        0: uniformed color
-			1: facet's color are computed with z*/ /* in case of a simple plot...!!! F.Leray 19.03.04 */
+		    0: uniformed color
+		    1: facet's color are computed with z*/ /* in case of a simple plot...!!! F.Leray 19.03.04 */
   sciTypeOf3D typeof3d;
   int hiddencolor;
   BOOL isselected;
@@ -937,6 +1029,11 @@ typedef struct
   /** specifies if this object is visble             */
   BOOL visible;
 
+  int flag_x;
+  int flag_y;
+  
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciSurface;  /** */
 
@@ -951,6 +1048,8 @@ typedef struct
   long *from_entity; /* vector of handles on the facet or segment entity*/
   int *index_in_entity;/*index of facet or segment in its entity*/
   int N; /* number of facets or segments */
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciMerge;  /** */
 
@@ -969,6 +1068,8 @@ typedef struct
   int isclip;
   double clip_region[4];
   int clip_region_set;
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciLightSource;	 /** */
 
@@ -1054,6 +1155,8 @@ typedef struct
   double clip_region[4]; /* to introduce for axis ? */
   int clip_region_set;
 
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciAxes;  
 
@@ -1081,7 +1184,7 @@ typedef struct
   double arrowsize;  /*F.Leray units : hundreds (i.e. 100, 150,...)*/
   double parfact;
   integer ptype; /* if ptype=0, it is segments; if ptype=1, it is champ (champ or champ1)*/
-  integer typeofchamp; /* when ptype=0, if typeofchamp=0 => champ is invoked else champ1 is invoked (typeofchamp==1) */
+  integer typeofchamp; /**< when ptype=0, if typeofchamp=0 => champ is invoked else champ1 is invoked (typeofchamp==1) */
   BOOL isselected;
   char *callback; /** specifies the text scilab code for the callback associated with this entity */
   int callbacklen; /** the length of the callback code */  
@@ -1094,6 +1197,8 @@ typedef struct
   double *vz; /**DJ.Abdemouche 2003**/
   double *vfz;
 
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciSegs; 
  
@@ -1127,6 +1232,8 @@ typedef struct
   int callbackevent;
   /** specifies if this object is visble             */
   BOOL visible;
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciGrayplot;  
 /** */
@@ -1161,6 +1268,8 @@ typedef struct
   int callbackevent;
   /** specifies if this object is visble             */
   BOOL visible;
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciFec;  /** */
 
@@ -1218,7 +1327,7 @@ sciLabelMenu;  /** */
 
 /**@name Menu
  * Structure used to specify a menu 
-  */
+ */
 typedef struct
 {
   sciRelationShip relationship;
@@ -1233,13 +1342,13 @@ sciMenu /** */ , sciMenuContext;/** */
 /**@name MenuContext
  * Structure used to specify popupmenu called with the right mouse button
  *
-typedef struct {
-  sciRelationShip    relationship;
-  sciGraphicContext graphiccontext;
-  sciText           text;
-  sciLabelMenu      *plabelmenu;
-} sciMenuContext ;
-ATTENTION DECLARE SI DESSUS */
+ typedef struct {
+ sciRelationShip    relationship;
+ sciGraphicContext graphiccontext;
+ sciText           text;
+ sciLabelMenu      *plabelmenu;
+ } sciMenuContext ;
+ ATTENTION DECLARE SI DESSUS */
 
 /**@name StatusBar
  * Structure used to specify 
@@ -1256,7 +1365,7 @@ sciStatusBar;  /** */
 
 /**@name Panner
  * Structure used to specify 
-  */
+ */
 typedef struct
 {
   sciRelationShip relationship;
@@ -1271,6 +1380,8 @@ typedef struct
   /** specifies if this object is visble             */
   BOOL visible;
 
+  int * user_data; /* adding 27.06.05 */
+  int size_of_user_data;
 }
 sciAgreg;  /** */
 
