@@ -1,8 +1,8 @@
       subroutine matnew
 c ====================================================================
 c
-c             controllabilite ,observabilite
-c             reponse frequentielle
+c             
+c    Frequency response, time response
 c
 c ====================================================================
 c
@@ -14,7 +14,7 @@ c
       integer iadr,sadr
 c
 c         fin    1     2      3      4       5     6
-c              contr   ppol  tzer    freq  ltitr  rtitr
+c              xxxx   ppol  tzer    freq  ltitr  rtitr
 c
       iadr(l)=l+l-1
       sadr(l)=(l/2)+1
@@ -33,188 +33,10 @@ c
       err=0
       goto (10,20,30,40,50,60) fin
  10   continue
-c
-c forme controllable
-c ==================
-c
-      if(rhs.lt.2.or.lhs.gt.4) then
-         call error(39)
-         return
-      endif
-      if(rhs.eq.3) then
-         ito=iadr(lstk(top))
-         lito=sadr(ito+4)
-         tol=stk(lito)
-         if(istk(ito+1)*istk(ito+2).eq.2) then
-           atol=stk(lito+1)
-         else 
-           atol=0.0d0
-         endif
-         top=top-1
-      else
-         tol=eps
-         atol=0.0d0
-      endif
-      mode=1
-      if(lhs.eq.1) mode=0
-c
-c     matrice b
-      ib=iadr(lstk(top))
-      if(istk(ib).ne.1) then
-         err=2
-         call error(53)
-         return
-      endif
-      if(istk(ib+3).ne.0) then
-         err=2
-         call error(52)
-         return
-      endif
-      mb=istk(ib+1)
-      nb=istk(ib+2)
-      nnb=nb*mb
-      lb=sadr(ib+4)
-c   matrice a
-      top=top-1
-      ia=iadr(lstk(top))
-      if(istk(ia).ne.1) then
-         err=1
-         call error(53)
-         return
-      endif
-      if(istk(ia+3).ne.0) then
-         err=1
-         call error(52)
-         return
-      endif
-      ma=istk(ia+1)
-      na=istk(ia+2)
-      if(na.ne.ma) then
-         err=1
-         call error(20)
-         return
-      endif
-      nna=ma*na
-      la=sadr(ia+4)
-c
-      if(nnb.eq.0) then
-         if(lhs.le.2) then
-c     n
-            il=iadr(lstk(top))
-            istk(il)=1
-            istk(il+1)=1
-            istk(il+2)=1
-            istk(il+3)=0
-            l=sadr(il+4)
-            stk(l)=0.0d0
-            lstk(top+1)=l+1
-            if(lhs.eq.2) then
-c     u
-               top=top+1
-               il=iadr(lstk(top))
-               istk(il)=1
-               istk(il+1)=na
-               istk(il+2)=na
-               istk(il+3)=0
-               l=sadr(il+4)
-               call dset(na*na,0.0d0,stk(l),1)
-               call dset(na,1.0d0,stk(l),na+1)
-               lstk(top+1)=l+na*na
-            endif
-         elseif(lhs.eq.4) then
-            top=top+1
-c     u
-            top=top+1
-            il=iadr(lstk(top))
-            istk(il)=1
-            istk(il+1)=na
-            istk(il+2)=na
-            istk(il+3)=0
-            l=sadr(il+4)
-            call dset(na*na,0.0d0,stk(l),1)
-            call dset(na,1.0d0,stk(l),na+1)
-            lstk(top+1)=l+na*na
-c     ind
-            top=top+1
-            il=iadr(lstk(top))
-            istk(il)=1
-            istk(il+1)=1
-            istk(il+2)=1
-            istk(il+3)=0
-            l=sadr(il+4)
-            stk(l)=0.0d0
-            lstk(top+1)=l+1
-         endif
-         return
-      endif
-c
-      if(mb.ne.na) then
-         call error(60)
-         return
-      endif
-c
-      if(dnrm2(nb*mb,stk(lb),1).lt.atol) then
-         call dset(nb*mb,0.0d0,stk(lb),1)
-      endif
-      iq=iadr(lb+nnb)
-      lq=sadr(iq+4)
-      lka=lq+nna
-      lk1=lka+na*nb
-      lk2=lk1+nb
-      lk3=lk2+nb
-      ilw=iadr(lk3+nb)
-      err=sadr(ilw+nb)-lstk(bot)
-      if(err.gt.0) then
-         call error(17)
-         return
-      endif
-      call ssxmc(na,nb,stk(la),na,stk(lb),ir,indcon,istk(ilw),stk(lq),
-     x           stk(lka),stk(lk1),stk(lk2),stk(lk3),tol,1)
-c    on stocke les resultats
-      if(lhs.gt.2) goto 11
-c index de controllabilite
-      istk(ia+1)=1
-      istk(ia+2)=1
-      stk(la)=dble(ir)
-      lstk(top+1)=la+1
-      if(mode.eq.0) return
-c     on sort aussi q
-      top=top+1
-      iq=iadr(lstk(top))
-      istk(iq)=1
-      istk(iq+1)=na
-      istk(iq+2)=na
-      istk(iq+3)=0
-      lqnew=sadr(iq+4)
-      call dcopy(nna,stk(lq),1,stk(lqnew),1)
-      lstk(top+1)=lqnew+nna
-      goto 99
-   11 if(lhs.ne.4) then
-         call error(39)
-         return
-      endif
-      top=top+2
-      iq=iadr(lstk(top))
-      istk(iq)=1
-      istk(iq+1)=na
-      istk(iq+2)=na
-      istk(iq+3)=0
-      lstk(top+1)=lq+nna
-      top=top+1
-      ils=iadr(lstk(top))
-      istk(ils)=1
-      istk(ils+1)=1
-      istk(ils+2)=indcon
-      istk(ils+3)=0
-      ls=sadr(ils+4)
-      do 12 i=1,indcon
-      stk(ls-1+i)=real(istk(ilw-1+i))
-   12 continue
-      lstk(top+1)=ls+indcon
       goto 99
 c
-c placement de poles ppol
-c=========================
+c  ppol
+c
    20 continue
       if(lhs.ne.1.or.rhs.ne.3) then
          call error(39)
@@ -303,6 +125,7 @@ c calcul de la forme canonique orthogonale
    22 call ssxmc(n,m,stk(la),n,stk(lb),idc,inc,istk(ilw),stk(lz),
      1  stk(lw+m),stk(lw+m+n*m),stk(lw+2*m+n*m),stk(lw),
      2    eps,1)
+      if(idc.lt.n) goto 91
 c placement de poles
       lw1=lw+m+m*m
       lw2=lw1+m*max(2,m)
@@ -311,7 +134,7 @@ c placement de poles
      2    stk(lw1),stk(lw2),stk(lw2+n),stk(lw2+2*n),stk(lw2+2*n+m))
       if(ierr.ne.0) goto 91
       istk(ia+1)=m
-      call dcopy(n*m,stk(lg),1,stk(la),1)
+      call unsfdcopy(n*m,stk(lg),1,stk(la),1)
       lstk(top+1)=la+n*m
       goto 99
 c
@@ -435,8 +258,8 @@ c     sortie des resultats
       istk(ia+1)=nu
       istk(ia+2)=min(nu,1)
       istk(ia+3)=1
-      call dcopy(nu,stk(lr),1,stk(la),1)
-      call dcopy(nu,stk(li),1,stk(la+nu),1)
+      call unsfdcopy(nu,stk(lr),1,stk(la),1)
+      call unsfdcopy(nu,stk(li),1,stk(la+nu),1)
       lstk(top+1)=la+nu+nu
       top=top+1
       ib=iadr(lstk(top))
@@ -445,7 +268,7 @@ c     sortie des resultats
       istk(ib+1)=nu
       istk(ib+2)=min(nu,1)
       istk(ib+3)=0
-      call dcopy(nu,stk(lk2),1,stk(lb),1)
+      call unsfdcopy(nu,stk(lk2),1,stk(lb),1)
       lstk(top+1)=lb+nu
       if(lhs.eq.3) then
          top=top+1
@@ -598,7 +421,7 @@ c
       if(rhs.eq.5) call dadd(p*q,stk(ld),1,stk(lwgr+ig),1)
    42 continue
       igran = (itfr+1)*p*q*ido
-      call dcopy(igran,stk(lwgr),1,stk(la),1)
+      call unsfdcopy(igran,stk(lwgr),1,stk(la),1)
       istk(ia) = 1
       istk(ia+1) = q
       istk(ia+2) = p*ido
@@ -702,7 +525,7 @@ c
       istk(il+2)=nd*nf
       istk(il+3)=itf
       l=sadr(il+4)
-      call dcopy(md*nd*nf*(itf+1),stk(lw),1,stk(l),1)
+      call unsfdcopy(md*nd*nf*(itf+1),stk(lw),1,stk(l),1)
       lstk(top+1)=l+md*nd*nf*(itf+1)
       goto 99
 c
@@ -763,6 +586,11 @@ c
          call error(52)
          return
       endif
+      if(istk(ifr+1).ne.istk(ib+2)) then
+         err=3
+         call error(89)
+         return
+      endif
       top=top-1
 c
       ia = iadr(lstk(top))
@@ -807,7 +635,7 @@ c
       lfr = sadr(ifr+4)
       if(rhs.eq.4) then
                    lx=sadr(ix+4)
-                   call dcopy(n,stk(lx),1,stk(lwgr),1)
+                   call unsfdcopy(n,stk(lx),1,stk(lwgr),1)
                    else
                    call dset(n,0.0d+0,stk(lwgr),1)
       endif
@@ -819,7 +647,7 @@ c
       call dadd(n,stk(lw),1,stk(lwgr+ig),1)
    52 continue
       if(lhs.eq.1) goto 53
-      call dcopy(n,stk(lwgr+ig),1,stk(la),1)
+      call unsfdcopy(n,stk(lwgr+ig),1,stk(la),1)
       lstk(top+1)=la+n
       istk(ia+1)=n
       istk(ia+2)=1
@@ -830,7 +658,7 @@ c
       istk(iy+2)=ido
       istk(iy+3)=0
       ly=sadr(iy+4)
-      call dcopy(ido*n,stk(lwgr),1,stk(ly),1)
+      call unsfdcopy(ido*n,stk(lwgr),1,stk(ly),1)
       lstk(top+1)=ly+n*ido
       goto 99
 c
@@ -1049,7 +877,7 @@ c
       istk(iy+2)=ido
       istk(iy+3)=0
       ly=sadr(iy+4)
-      call dcopy(ido*nd,stk(lr),1,stk(ly),1)
+      call unsfdcopy(ido*nd,stk(lr),1,stk(ly),1)
       lstk(top+1)=ly+nd*ido
 99    return
       end
