@@ -35,10 +35,10 @@ c
          call basout(io,wte,' matopt '//buf(1:4))
       endif
 c     
-c     optim   quapro   semidef fsolve lsqrsolve
-c     1        2        3        4       5
+c     optim   quapro   semidef fsolve lsqrsolve qld
+c     1        2        3        4       5      6
 c     
-      goto(01,400,500,600,700 ) fin
+      goto(01,400,500,600,700,800 ) fin
 c     
       
 c     optim
@@ -367,7 +367,7 @@ c     on initialise nizs,nrzs,ndzs
             call boptim(indsim,nx,stk(lx),stk(lf),stk(lg),
      &           izs,rzs,dzs)
          endif
-         if(err.gt.0) return
+         if(err.gt.0.or.err1.gt.0) return
 c     
          if(indsim.le.0) then
             indopt=-7
@@ -549,7 +549,7 @@ c
             call boptim(indsim,nx,stk(lx),stk(lf),stk(lg),
      &           istk(lizs),sstk(lrzs),stk(ldzs))
          endif
-         if(err.gt.0)return
+         if(err.gt.0.or.err1.gt.0)return
          if(indsim.le.0) then
             indopt=-7
             if(indsim.eq.0) indopt=0
@@ -596,7 +596,7 @@ c
      &           stk(lvar),epsg,mode,itmax,napm,imp,io,stk(ltv),
      &           istk(lizs),sstk(lrzs),stk(ldzs))
          endif
-         if(err.gt.0) return
+         if(err.gt.0.or.err1.gt.0) return
 c     affectation de indopt
          epsg=sqrt(epsg)
          indopt=9
@@ -765,7 +765,7 @@ c     optimiseur qnbd
      &           stk(ltv),ntv,istk(litv),nitv,
      &           istk(lizs),sstk(lrzs),stk(ldzs))
          endif
-         if(err.gt.0) return
+         if(err.gt.0.or.err1.gt.0) return
          go to 300
       endif
 c     
@@ -806,7 +806,7 @@ c     optimiseur gcbd
      &           stk(ltv),ntv,istk(litv),nitv,
      &           istk(lizs),sstk(lrzs),stk(ldzs))
          endif
-         if(err.gt.0) return
+         if(err.gt.0.or.err1.gt.0) return
          go to 300
       endif
 c     
@@ -1228,12 +1228,14 @@ c
          call error(17)
          return
       endif
-c     
-c     call PLCBAS(H,P,C,D,CI,CS,IRA,MI,MD,X,F,W,IV,LAGR,IMP,IO,N,
-c     &                  MODO,INFO)
+
+C     SUBROUTINE PLCBAS(H,P,C,D,CI,CS,IRA,MI,MD,X,F,W,IV,DLAGR,IMP,IO,N,
+C    &                  MODO,INFO,ITER)
+C
+      iter=0
       call plcbas(stk(l2),stk(l3),stk(l4),stk(l5),stk(l6),stk(l7),
      &     ira,mi,md,stk(lw9),stk(lw10),stk(lw11),stk(lw12),
-     &     stk(lw13),imp,wte,n1,modo,info)
+     &     stk(lw13),imp,wte,n1,modo,info,iter)
       if (info .ne. 0) then
          if (info .eq. 1) then
             call msgs(104,0)
@@ -1531,6 +1533,12 @@ c
  700  call intlsqrsolve("lsqrsolve")
       return
 
+c
+c     qld
+c     
+ 800  call intqld("qld")
+      return
+
 
       end
 
@@ -1634,7 +1642,7 @@ C     -------------------------------------------
          call hybrd1(bsolv,mn1,stk(lr1),stk(lw4),tol,info,
      $        stk(lw7),nn7)
       endif 
-      if(err.gt.0) return
+      if(err.gt.0.or.err1.gt.0) return
 c     
       top=top-1
       if(lhs.eq.1) then
