@@ -62,8 +62,8 @@ c     lasterror version loadhistory savehistory gethistory resethistory macr2tre
 c     53         54        55          56         57          58        59
 c     sleep getos
 c     60    61 
-c     banner fromjava calendar
-c     62     63       64
+c     banner fromjava Calendar getmemory fromc
+c     62     63       64        65       66
       if (ddt .eq. 4) then
          write(buf(1:4),'(i4)') fin
          call basout(io,wte,' matsys '//buf(1:4))
@@ -78,7 +78,7 @@ c
      +      450,500,510,600,610,620,630,640,650,660,
      +      670,680,681,682,683,684,690,691,692,693,
      +      694,695,697,698,699,700,701,702,703,
-     +      705,706,707,708,709),fin
+     +      705,706,707,708,709,710,711),fin
 c     
 c     debug
  10   call intdebug()
@@ -271,8 +271,12 @@ c     mtlb_mode
       goto 999     
  708  call intfromjava('fromjava')
       goto 999
- 709  call intcalendar('calendar')
+ 709  call intcalendar('Calendar')
       goto 999
+ 710  call intgetmemory('getmemory')
+      goto 999
+ 711  call intfromc('fromc')
+      goto 999   
  998  continue
 c     fake calls : only to force the 
 c     linker to load the following functions
@@ -1702,11 +1706,19 @@ c     Copyright INRIA
       include '../stack.h'
       integer topk,p
       logical checkrhs,checklhs,getscalar,cremat,getsmat,checkval
+      character*9 opt
+      character*9 pause,cont,kill,stp,nomess
       integer iadr
 c
       data local/21/
 c    
       iadr(l)=l+l-1
+      pause ='pause    '
+      cont  ='continue '
+      kill  ='kill     '
+      stp   ='stop     '
+      nomess='nomessage'
+
 c
       topk=top
       rhs=max(0,rhs)
@@ -1724,28 +1736,47 @@ c
       num=0
       imode=0
       imess=0
+
+
       do 201 i=1,rhs
+         top=topk-rhs+i
          il=iadr(lstk(top))
          if(istk(il).lt.0) il=iadr(istk(il+1))
-
-         if(istk(il).eq.1) then
+         if(istk(il).eq.1.and.i.eq.1) then
             if(.not.getscalar('errcatch',topk,top,l)) return
             num=nint(stk(l))
-         else if(istk(il).eq.10) then
+            if(dfloat(num).ne.stk(l).or.num.lt.-1) then
+               err=i
+               call error(44)
+               return
+            endif
+         else if(istk(il).eq.10.and.i.gt.1) then
             if(.not.getsmat('errcatch',topk,top,m,n,1,1,il1,n1)) return
             if(.not.checkval('errcatch',m*n,1) ) return
-            l=abs(istk(il1))
-            if(l.eq.12) imode=1
-            if(l.eq.25) imode=2
-            if(l.eq.23) imess=1
-            if(l.eq.28) imode=3
+            n1=min(n1,9)
+            call cvstr(n1,istk(il1),opt,1)
+            if(opt(1:n1).eq.cont(1:n1)) then
+               imode=1
+            elseif(opt(1:n1).eq.pause(1:n1)) then
+               imode=2
+            elseif(opt(1:n1).eq.nomess(1:n1)) then
+               imess=1
+            elseif(opt(1:n1).eq.stp(1:n1)) then
+               imode=3
+            elseif(opt(1:n1).eq.kill(1:n1)) then
+               imode=0
+            else
+               err=i
+               call error(44)
+               return
+            endif
          else
-            err=rhs-1+i
+            err=i
             call error(44)
             return
          endif
-         top=top-1
  201  continue
+      top=topk-rhs
       errct=(8*imess+imode)*100000+abs(num)
       catch=max(imode,1)
       if(num.lt.0) errct=-errct
@@ -1812,19 +1843,21 @@ c     .  error(str [,errn])
 c     .  get error number  if given
             if(.not.getscalar('error',topk,top,l)) return
             errn=int(stk(l))
-            if(errn.ge.100000) then
-               err=2
-               call error(116)
-               return
-            endif
             top=top-1
          else
-            errn=9999
+c Bug 1421 corrected - Francois VOGEL June 2006
+            errn=10000
          endif
          if(.not.getsmat('error',topk,top,m,n,1,1,il,n1)) return
          buf=' '
          call cvstr(n1,istk(il),buf,1)
       endif
+
+      if(errn.le.0) then
+         err=2
+         call error(116)
+         return
+      endif
       call error(errn)
       return
       end
@@ -2968,39 +3001,6 @@ c
       stk(l)=dble(k)
       return
       end
-      subroutine intwhat(fname)
-c     
-c     Copyright INRIA
-      include '../stack.h'
-c     
-      character*(*) fname
-      logical checklhs,checkrhs
-      integer iadr
-
-      integer cmdl,id(nsiz)
-      parameter (cmdl = 27)
-      integer cmd(nsiz,cmdl)
-      common/cmds/cmd
-
-
-      iadr(l)=l+l-1
-c
-      rhs=max(0,rhs)
-c
-      if(.not.checklhs(fname,1,1)) return
-      if(.not.checkrhs(fname,0,0)) return
-c
-      call funtab(id,0,0)
-c     comandes
-      fin = 1
-      call msgs(41,0)
-      call prntid(cmd,cmdl,wte)
-c     
-      top=top+1
-      il = iadr(lstk(top))
-      istk(il) = 0
-      lstk(top+1) = lstk(top) + 1
-      end
       subroutine intwhereis
 c     Copyright INRIA
       include '../stack.h'
