@@ -3,9 +3,11 @@
 c     Copyright INRIA
       include '../stack.h'
       logical opened,ptover,cremat
-      integer fd,vol,top0,srhs
+      integer fd,vol,top0,srhs,bl(nsiz)
       double precision res
+      logical eqid
       integer iadr,sadr
+      data bl/nsiz*673720360/
 c     
       iadr(l)=l+l-1
       sadr(l)=(l/2)+1
@@ -45,12 +47,25 @@ c     loop on variables to save
  20   k=k+1
       il=iadr(lstk(k))
       vol=lstk(k+1)-lstk(k)
- 21   call savevar(fd,idstk(1,k),il,vol,ierr)
+ 21   continue
+
+      if(eqid(idstk(1,k),bl)) then
+         err=k
+         call error(248)
+         if (.not.opened) call mclose (fd,res)
+         return
+      endif
+      call savevar(fd,idstk(1,k),il,vol,ierr)
+      if(ierr.ne.0) then
+         call error(263)
+         if (.not.opened) call mclose (fd,res)
+         return
+      endif
       if(fun.ge.0) goto 25
 c     overloaded save function
       if ( ptover(1,psiz)) return
       ilrec         = iadr(lstk(top+1))
-      err=sadr(ilrec+7)-lstk(bot)
+      err=sadr(ilrec+8)-lstk(bot)
       if(err.gt.0) then
          call error(17)
          return
@@ -62,6 +77,11 @@ c     overloaded save function
       istk(ilrec+4) = k
       istk(ilrec+5) = top0
       istk(ilrec+6) = vol
+      if(opened) then
+         istk(ilrec+7) = 1
+      else
+         istk(ilrec+7) = 0
+      endif
       lstk(top+1)=sadr(ilrec+7)
       rstk(pt)=905
       pstk(pt)=ilrec
@@ -86,6 +106,7 @@ c     *call* parse
       k          = istk(ilrec+4)
       top0       = istk(ilrec+5)
       vol        = istk(ilrec+6)
+      opened     = (istk(ilrec+7).eq.1)
       pt=pt-1 
       if(rstk(pt).eq.911) goto 21
 
@@ -110,10 +131,10 @@ c     (id1(1).ne.blank) and if this variable if loaded k1 ist set to its
 c     index in the stack
 c
       include '../stack.h'
-      integer id1(nsiz),k1
+      integer id1(nsiz),k1,vol
       logical opened,cremat,ptover,eqid
       integer fd,id(nsiz),semi,blank,top0,endian,getendian,it,ssym
-      double precision res
+      double precision res,offset
       integer iadr,sadr
       data semi/43/,blank/40/
 c     
@@ -145,6 +166,7 @@ c     compatibility (file opened by fortran)
       endif
 c     test for compatibility
       endian=getendian()
+      call mtell(fd,offset,ierr)
       call mgetnc (fd,it,1,'il'//char(0),ierr)
       if(endian.eq.1.and.it.eq.28.or.
      $     endian.eq.0.and.it.eq.469762048) then
@@ -154,7 +176,7 @@ c     .  old mode
          k1=fin
          return
       else
-         call mseek(fd,0,'set'//char(0),ierr)
+         call mseek(fd,int(offset),'set'//char(0),ierr)
       endif
 
       
@@ -187,9 +209,9 @@ c     .  get table of requested variable names
       top=top+1
 
 c     load all variables stored in a file
-      il=iadr(lstk(top))
       kvar=0
  10   continue
+      il=iadr(lstk(top))
  21   call loadvar(fd,id,il,nn,ierr)
       if(ierr.gt.0) return
       if(ierr.lt.0) goto 50
@@ -198,11 +220,33 @@ c     load all variables stored in a file
          goto 26
       endif
 c     overloaded save function
+
+c     preserve context
+      if ( ptover(1,psiz)) return
+      rstk(pt)=0
+      pstk(pt)=ilt
+      call putid(ids(1,pt),id)
       if ( ptover(1,psiz)) return
       rstk(pt)=906
       pstk(pt)=rhs
       ids(1,pt)=kvar
       ilv=-fun
+c     ilv points to the beginning of the variable to load
+c     store it into ids (used by the function called)
+      ids(2,pt)=ilv
+c     preserve variable type
+      ids(3,pt)=istk(ilv)
+c     set the end of the variable temporarily 
+      lstk(top+1)=sadr(ilv)
+c     preserve value of top
+      ids(5,pt)=top
+      if(opened) then
+         ids(6,pt)=fd
+      else
+         ids(6,pt)=-fd
+      endif
+
+c     form name of the function to call
       call funnam(ids(1,pt+1),'load',ilv)
 c     create a variable with fd
       top=top+1
@@ -213,12 +257,40 @@ c     create a variable with fd
       return
 c     *call* parse
  24   continue
+c
+c     restore context
       rhs=pstk(pt)
       kvar=ids(1,pt)
+      if (ids(6,pt).gt.0) then
+         opened=.true.
+         fd=ids(6,pt)
+      else
+         opened=.false.
+         fd=-ids(6,pt)
+      endif
+      if(ids(5,pt).eq.top) then
+c     .  load has been done by a primitive
+         nn1=ids(4,pt)
+      else
+c     .  load has been done by a scilab function
+         nn1=iadr(lstk(top+1))-iadr(lstk(top))
+         top=top-1
+      endif
+      ilv=ids(2,pt)
+      istk(ilv)=ids(3,pt)
       pt=pt-1 
-      if(rstk(pt).eq.912) goto 21
-      goto 26
-
+      ilt=pstk(pt)
+      call putid(id,ids(1,pt))
+      pt=pt-1
+      if(rstk(pt).eq.912) then
+c     .  a sublist
+         ids(5,pt)=nn1
+         goto 21
+      else
+c     .   a simple variable set the end
+         lstk(top+1)=sadr(ilv+nn1)
+         goto 26
+      endif
  26   continue
       if(rhs.gt.1) then
 c     .  check if loaded variable is required
@@ -340,16 +412,14 @@ c
 c
 c
       iadr(l)=l+l-1
-c      sadr(l)=(l/2)+1
+      sadr(l)=(l/2)+1
 c
 
       fmti='il'//char(0)
       fmtd='dl'//char(0)
 c
       if(rstk(pt).eq.912) then
-         il1=il
-         if(istk(il1).lt.0) il1=iadr(istk(il1+1))
-         call loadlist(fd,il1,nn,ierr)
+         call loadlist(fd,il,nn,ierr)
          return
       endif
 c
@@ -360,7 +430,6 @@ c     read id and type
       if(ierr.ne.0) return
       call mgetnc (fd,istk(il1),1,fmti,ierr)
       if(ierr.ne.0) return
-
       if(istk(il1).eq.1) then
          call loadmat(fd,il1,nn,ierr)
       elseif(istk(il1).eq.2.or.istk(il1).eq.129) then
@@ -389,6 +458,7 @@ c     read id and type
          call loadptr(fd,il1,nn,ierr)
       else
          fun=-il1
+         lstk(top+1)=sadr(il1)
       endif
       if(err.gt.0) ierr=1
       return
@@ -471,6 +541,7 @@ c     .  a sublist
          il=il1
          goto 10
       elseif(istk(il1).eq.128) then 
+         
          call saveptr(fd,il1,ierr)
       else
 c     .  call an external function
@@ -518,6 +589,7 @@ c
 c
       fmti='il'//char(0)
       fmtd='dl'//char(0)
+      ierr=0
 c
       il1=il
       if(rstk(pt).eq.912) then
@@ -526,19 +598,14 @@ c     .  manage recursion
          il=ids(2,pt)
          i=ids(3,pt)
          il0=ids(4,pt)
+         nne=ids(5,pt)
          pt=pt-1
          l=sadr(il+n+3)
-         il1=iadr(l-1+istk(il+1+i))
-         lt=lstk(top)
-         l1=sadr(il1)
-         nne=lstk(top+1)-lt
-         if(l1.ne.lt) call dcopy(lstk(top+1)-lt,stk(lt),1,stk(l1),1)
-         istk(il+2+i)=istk(il+1+i)+nne
-         nne=2*nne
+         il1=iadr(lstk(top))
+         istk(il+2+i)=istk(il+1+i)+lstk(top+1)-lstk(top)
          top=top-1
          goto 20
       endif
-
       
  10   il0=il
 c     read list header without type
@@ -568,7 +635,6 @@ c     read the elements
       if(i.gt.n) goto 30
       if(istk(il+2+i)-istk(il+1+i).eq.0) goto 20
       il1=iadr(l-1+istk(il+1+i))
-
 c     read  type
       call mgetnc (fd,istk(il1),1,fmti,ierr)
       if(ierr.ne.0) return
@@ -586,7 +652,7 @@ c     read  type
       elseif(istk(il1).eq.7) then
          call loadmsp(fd,il1,nne,ierr)
       elseif(istk(il1).eq.8) then
-         call loadint(fd,il1,nn,ierr)
+         call loadint(fd,il1,nne,ierr)
       elseif(istk(il1).eq.10) then
          call loadstr(fd,il1,nne,ierr)
       elseif(istk(il1).eq.11) then
@@ -615,6 +681,9 @@ c     .  call an external function
          ids(2,pt)=il
          ids(3,pt)=i
          ids(4,pt)=il0
+         lstk(top+1)=sadr(il1)
+         top=top+1
+         lstk(top+1)=lstk(top)
          fun=-il1
 c     *call* parse
          return
@@ -641,6 +710,7 @@ c
  40   continue
 c     finish
       nn=il1+nne-il0
+      il=il0
       return
       end
 
@@ -653,7 +723,7 @@ c
       character*3 fmti,fmtd
       integer sadr
 c
-c      iadr(l)=l+l-1
+      iadr(l)=l+l-1
       sadr(l)=(l/2)+1
 c
       fmti='il'//char(0)
@@ -664,7 +734,8 @@ c     write matrix header type excluded
       if(ierr.ne.0) return
 c     write matrix elements
       mn=istk(il+1)*istk(il+2)*(istk(il+3)+1)
-      call mputnc(fd,stk(sadr(il+4)),mn,fmtd,ierr)
+      l=sadr(il+4)
+      call mputnc(fd,istk(iadr(l)),mn,fmtd,ierr)
       return
       end
 
@@ -698,7 +769,8 @@ c     read matrix elements
          return
       endif
       l=sadr(il+4)
-      call mgetnc(fd,stk(l),mn,fmtd,ierr)
+      call mgetnc(fd,istk(il+4),mn,fmtd,ierr)
+c      call mgetnc(fd,stk(l),mn,fmtd,ierr)
       n=iadr(l+mn)-il
 c      n=4+2*mn
       return
@@ -713,7 +785,7 @@ c
       character*3 fmti,fmtd
       integer sadr
 c
-c      iadr(l)=l+l-1
+      iadr(l)=l+l-1
       sadr(l)=(l/2)+1
 c
       fmti='il'//char(0)
@@ -725,7 +797,8 @@ c     write matrix header without type
       if(ierr.ne.0) return
 c     write polynomials coefficients
       mn1=(istk(il+8+mn)-1)*(istk(il+3)+1)
-      call mputnc(fd,stk(sadr(il+9+mn)),mn1,fmtd,ierr)
+      l=sadr(il+9+mn)
+      call mputnc(fd,istk(iadr(l)),mn1,fmtd,ierr)
       return
       end
 
@@ -770,7 +843,7 @@ c     read polynomials coefficients
          return
       endif
       l=sadr(il+9+mn)
-      call mgetnc(fd,stk(l),mn1,fmtd,ierr)
+      call mgetnc(fd,istk(iadr(l)),mn1,fmtd,ierr)
       n=iadr(l+mn1)-il
 c      n=9+mn+2*mn1
       return
@@ -1076,7 +1149,7 @@ c
       character*3 fmti,fmtd
       integer sadr
 c
-c      iadr(l)=l+l-1
+      iadr(l)=l+l-1
       sadr(l)=(l/2)+1
 c
       fmti='il'//char(0)
@@ -1091,7 +1164,8 @@ c     write matrix elements
       call mputnc(fd,istk(il+5),m+nel,fmti,ierr)
       if(ierr.ne.0) return
       mn=nel*(istk(il+3)+1)
-      call mputnc(fd,stk(sadr(il+5+m+nel)),mn,fmtd,ierr)
+      l=sadr(il+5+m+nel)
+      call mputnc(fd,istk(iadr(l)),mn,fmtd,ierr)
       return
       end
 
@@ -1138,7 +1212,7 @@ c     read matrix elements
          return
       endif
       l=sadr(il+5+m+nel)
-      call mgetnc(fd,stk(l),mn,fmtd,ierr)
+      call mgetnc(fd,istk(iadr(l)),mn,fmtd,ierr)
       n=iadr(l+mn)-il
 c      n=5+m+nel+2*mn
       return
@@ -1278,7 +1352,7 @@ c
       character*3 fmti,fmtd
       integer sadr
 c
-c      iadr(l)=l+l-1
+      iadr(l)=l+l-1
       sadr(l)=(l/2)+1
 c
       fmti='il'//char(0)
@@ -1293,7 +1367,8 @@ c     write matrix elements
       call mputnc(fd,istk(il+5),n+nel+1,fmti,ierr)
       if(ierr.ne.0) return
       mn=nel*(istk(il+3)+1)
-      call mputnc(fd,stk(sadr(il+6+n+nel)),mn,fmtd,ierr)
+      l=sadr(il+6+n+nel)
+      call mputnc(fd,istk(iadr(l)),mn,fmtd,ierr)
       return
       end
 
@@ -1341,7 +1416,7 @@ c     read matrix elements
          return
       endif
       l=sadr(il1)
-      call mgetnc(fd,stk(l),mn,fmtd,ierr)
+      call mgetnc(fd,istk(iadr(l)),mn,fmtd,ierr)
       n=iadr(l+mn)-il
 c      n=5+n+nel+2*mn
       return
@@ -1362,12 +1437,16 @@ c
       fmti='il'//char(0)
       fmtd='dl'//char(0)
 c
-c     write  header type excluded
-      call mputnc (fd,istk(il+1),3,fmti,ierr)
-      if(ierr.ne.0) return
-c     write pointer
-      call mputnc(fd,stk(sadr(il+4)),1,fmtd,ierr)
+      buf='handle to sparse lu factors cannot be saved yet'
+      call error(997)
+      ierr=997
       return
+c     write  header type excluded
+c      call mputnc (fd,istk(il+1),3,fmti,ierr)
+c      if(ierr.ne.0) return
+c     write pointer
+c      call mputnc(fd,stk(sadr(il+4)),1,fmtd,ierr)
+c      return
       end
 
       subroutine loadptr(fd,il,n,ierr)
@@ -1394,7 +1473,7 @@ c     read matrix header without type
       if(ierr.ne.0) return
 c     read pointer
       l=sadr(il+4)
-      call mgetnc(fd,stk(l),1,fmtd,ierr)
+      call mgetnc(fd,istk(iadr(l)),1,fmtd,ierr)
       n=iadr(l+1)-il
 c      n=4+2*1
       return
@@ -1442,6 +1521,8 @@ c     Copyright INRIA
 c     Save a pointer on sparse lu factorization
       include '../stack.h'
       integer fd
+      external memused
+      integer memused
       character*3 fmti,fmtd
       integer sadr
 c
@@ -1468,98 +1549,19 @@ c     read matrix header without type
       endif
       if(it.eq.4) then
          call mgetnc(fd,istk(il+4),mn,fmti,ierr)
-         n=4+mn
       elseif(it.eq.2) then
          call mgetnc(fd,istk(il+4),mn,'sl'//char(0),ierr)
-         n=4+(mn+1)/2
       elseif(it.eq.1) then
          call mgetnc(fd,istk(il+4),mn,'c'//char(0),ierr)
-         n=4+(mn+3)/4
       elseif(it.eq.14) then
          call mgetnc(fd,istk(il+4),mn,'uil'//char(0),ierr)
-         n=4+mn
       elseif(it.eq.12) then
          call mgetnc(fd,istk(il+4),mn,'usl'//char(0),ierr)
-         n=4+(mn+1)/2
       elseif(it.eq.11) then
          call mgetnc(fd,istk(il+4),mn,'uc'//char(0),ierr)
-         n=4+(mn+3)/4
       endif
+      n=4+memused(it,mn)
       return
       end
 
 
-      subroutine v2cunit(k,mode,fd,opened,ierr)
-c     given variable #k (scalar or string) and mode 
-c     v2unit return a  logical unit attached to corresponding file
-
-      INCLUDE '../stack.h'
-c
-      logical opened
-      integer fd
-      character*2 mode
-      character*3 status
-      double precision w
-      integer iadr,sadr
-c     
-      iadr(l)=l+l-1
-      sadr(l)=(l/2)+1
-c
-      ierr=0
-      il=iadr(lstk(k))
-      if(istk(il).lt.0) il=iadr(istk(il+1))
-      if (istk(il).eq.1) then
-         fd = int(stk(sadr(il+4)))
-         if (istk(il+1)*istk(il+2).ne.1.or.istk(il+3).ne.0.or.
-     $        fd .lt. 0) then
-            err=1
-            ierr=1
-            call error(36)
-            return
-         endif
-         call getfileinfo(fd,ifa,iswap,ltype,mod,buf,lb,info)
-         if(info.eq.0.and.ltype.eq.1) then
-c     ierr=-1 line used for compatibility instead of error
-            ierr=-1
-            opened=.true.
-c            call error(244)
-            return
-         endif
-         opened=.true.
-      elseif(istk(il).eq.10) then
-         if(istk(il+1)*istk(il+2).ne.1) then
-            err=1
-            ierr=1
-            call error(36)
-            return
-         endif
-         mn=istk(il+5)-1
-         call cvstr(mn,istk(il+5+istk(il+1)*istk(il+2)),buf,1)
-         call cluni0(buf(1:mn), buf(mn+2:),mn1)
-         lunit = 0
-         buf(mn+mn1+2:mn+mn1+2)=char(0)
-         l=lnblnk(mode)
-         call mopen(fd,buf(mn+2:),mode(1:l)//char(0),1,w,ierr)
-
-         if(ierr.gt.0) then
-            if(ierr.eq.2) then
-               if(mode(1:1).eq.'r') then
-                  err=241
-               else
-                  err=240
-               endif
-            elseif(ierr.eq.1) then
-               err=66
-            endif
-            buf(mn+1:)=' '
-            call error(err)
-            return
-         endif
-         opened=.false.
-      else
-         err=1
-         ierr=1
-         call error(36)
-         return
-      endif
-      end
