@@ -9,14 +9,21 @@
 
 #include <string.h>
 #include <stdio.h>
+
 #include "../stack-c.h" 
-#include "../sun/men_Sutils.h" 
+
+#ifdef WIN32
+#include "../os_specific/win_mem_alloc.h" /* MALLOC */
+#else
+#include "../os_specific/sci_mem_alloc.h" /* MALLOC */
+#endif
+#include "../os_specific/men_Sutils.h" 
+
 
 #ifdef WIN32
 #define abs(x) ((x) >= 0 ? (x) : -(x)) /* pour abs  C2F(mvfromto) line 2689 */
 #endif
 /* Table of constant values */
-
 static integer cx1 = 1;
 static integer cx0 = 0;
 
@@ -1499,6 +1506,43 @@ int C2F(getrhscvar)(number, typex, it, m, n, lr, lc, type_len)
   return TRUE_;
 }
 
+/*---------------------------------------------------------------------
+ * elementtype:
+ *   returns the type of the element indexed by *number in the list 
+ *   whose variable number is *lnumber. If the indexed element does not exist
+ *   the function returns 0.
+ *---------------------------------------------------------------------*/
+
+int C2F(elementtype)(integer *lnumber, integer *number)
+{
+  integer il,lw,itype,n,ix,ili;
+  char *fname = Get_Iname();
+
+  if (*lnumber > Rhs) {
+    Scierror(999,"%s: bad call to elementtype! \r\n",fname);
+    return FALSE_;
+  }
+
+  lw = *lnumber + Top - Rhs; /*index of the variable numbered *lnumber in the stack */
+  il = iadr(*Lstk(lw)); 
+  if (*istk(il) < 0) il = iadr(*istk(il + 1));
+  itype = *istk(il ); /* type of the variable numbered *lnumber */
+  if (itype < 15 || itype > 17) { /* check if it is really a list */
+    Scierror(210,"%s: Argument %d: wrong type argument, expecting a list\r\n",fname,*lnumber);
+    return FALSE_;
+  }
+  n = *istk(il + 1);/* number of elements in the list */
+  itype = 0; /*default answer if *number is not a valid element index */
+  if (*number<=n && *number>0) {
+    ix = sadr(il + 3 + n); /* adress of the first list element */
+    if (*istk(il + 1+ *number) < *istk(il + *number + 2)) { /* the required element is defined */
+      ili = iadr(ix + *istk(il + 1+ *number) - 1); /* adress of the required element */
+      itype = *istk(ili);
+    }
+  }
+  return itype;
+}
+
 /*---------------------------------------------------------------------
  *     This function must be called after getrhsvar(lnumber,'l',...) 
  *     Argument lnumber is a list 
@@ -3166,6 +3210,23 @@ void *GetData(lw)
   return loci;
 }
 
+int GetDataSize(lw)
+     /* get memory used by the argument lw in double world etc */
+     int lw;
+{
+  int lw1 = lw + Top - Rhs ;
+  int l1 = *Lstk(lw1);
+  int *loci = (int *) stk(l1);
+  int n =  *Lstk(lw1+1)-*Lstk(lw1);
+  if (loci[0] < 0) 
+    {
+      l1 = loci[1];
+      loci = (int *) stk(l1);
+      n=loci[3];
+    }
+  return n;
+}
+
 void *GetRawData(lw)
      /* same as GetData BUT does not go to the pointed variable if lw is a reference */
      int lw;
@@ -3270,9 +3331,9 @@ static int intersci_push()
   int i;
   intersci_list *loc;
   intersci_state *new ;
-  new = malloc( Nbvars * sizeof(intersci_state) );
+  new = MALLOC( Nbvars * sizeof(intersci_state) );
   if (new == 0 ) return 0;
-  loc = malloc( sizeof(intersci_list) );
+  loc = MALLOC( sizeof(intersci_list) );
   if ( loc == NULL ) return 0;
   loc->next = L_intersci;
   loc->state = new; 
