@@ -17,6 +17,10 @@
 #define TRUE 1
 #define FALSE 0
 /*-----------------------------------------------------------------------------------*/
+void DisableInteractiveMode(void);
+/* Disable TCL/TK and graphic interfaces */
+/* Scilab no GUI no TCL/TK "kernel mode" */
+
 int StartScilab(char *SCIpath,char *ScilabStartup,int *Stacksize);
 /* Initialize Scilab */
 /* returns TRUE if it is OK else FALSE */
@@ -33,6 +37,12 @@ int SendScilabJob(char *job);
 /* Send a job to scilab */
 /* returns error code operation */
 
+void ScilabDoOneEvent(void);
+/* This procedure is the entry point to Scilab's event loop */
+
+int ScilabHaveAGraph(void);
+/* Returns TRUE if a graphic windows is opened */
+
 /*-----------------------------------------------------------------------------------*/
 #endif /* __CALLSCILAB__ */
 /*-----------------------------------------------------------------------------------*/
