@@ -18,7 +18,8 @@ c     de-activate output control
       lct(1)=0
 c
       errtyp=0
-      if(err1.eq.0.and.err2.eq.0) then
+c      if(err1.eq.0.and.err2.eq.0) then
+      if(err1.eq.0) then
 c     . locate the error in the current statement
         if(trace) call errloc(n)
 c     . output error message
@@ -78,6 +79,10 @@ c        . looking if error has occurred in execstr deff getf or comp
 c     .     error has occurred in an external
             errtyp=0
             pt0=p
+         elseif(rstk(p).eq.808) then
+c     .     error has occurred in a try instructions
+            errtyp=0
+            pt0=p
          elseif(rstk(p).eq.501.and.catch.eq.0) then
             if (rstk(p-1).eq.909) then
 c     .        exec of a function
@@ -164,12 +169,12 @@ c     recoverable by errcatch or not:
 c     errtyp=0  : recoverable error
 c     errtyp=1  : unrecoverable error
 c
-c     n : error number, if n execeeds the maximum error number this
+c     n : error number, if n exceeds the maximum error number this
 c         routines displays the error message contained in buf
 c!
       include '../stack.h'
       parameter (iif=1,iwhile=2,iselect=3)
-      integer n,errtyp
+      integer n,errtyp,bufl
       integer lunit,sadr,nl,io
       character line*340
 c
@@ -181,7 +186,7 @@ c
 c
 c Bug 1069 corrected - Francois VOGEL November 2004
       call linestore(0)
-      call funnamestore('',0)
+      call funnamestore(' ',0)
       call freemsgtable()
       call errstore(n)
       goto (
@@ -194,7 +199,7 @@ c Bug 1069 corrected - Francois VOGEL November 2004
      +  91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,104,105,
      + 106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,
      + 121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,
-     + 136,137,138,139,140,141,142,143
+     + 136,137,138,139,140,141,142,143,144
      + ),n
 
       goto (
@@ -207,7 +212,8 @@ c Bug 1069 corrected - Francois VOGEL November 2004
      +     260,261,262,263,264,265,266,267,268,269,
      +     270,271,272,273,274,275,276,277,278,279,
      +     280),n-199
-      if(n.ge.10000) return
+c Bug 1421 corrected - Francois VOGEL June 2006
+c     if(n.gt.10000) return
       goto 998
 c
     1 call msgout(io,lunit,'incorrect assignment')
@@ -824,6 +830,14 @@ c
  143  call msgout(io,lunit,
      +   'optim : df0 must be positive !')
       goto 999
+ 144  continue
+      call cvname(ids(1,pt+1),buf,1)
+      ilb=lnblnk(buf(1:nlgh))
+      call msgout(io,lunit,'Undefined operation for the given operands')
+      call msgout(io,lunit,'  check or define function '//buf(1:ilb)//
+     +  ' for overloading')
+      goto 999
+
  200  continue
       goto 999
 c     messages from 201 to 203 and 205 to 214 are no more used by error 
@@ -1101,10 +1115,11 @@ C     errors from semidef
       goto 999
  246  continue
       call cvname(ids(1,pt+1),buf,1)
+      ilb=lnblnk(buf(1:nlgh))
       call msgout(io,lunit,'function not defined for'//
-     &     ' given argument type(s)')
-      call msgout(io,lunit,'Check arguments or define function '
-     &     //buf(1:nlgh)//' for overloading')
+     &     ' given argument type(s),')
+      call msgout(io,lunit,'  check arguments or define function '
+     &     //buf(1:ilb)//' for overloading')
       goto 999
  247  continue
       goto 999
@@ -1255,6 +1270,7 @@ C     errors from semidef
       goto 999
  276  continue
       call msgout(io,lunit, 'Missing operator, comma, or semicolon')
+      errtyp=1
       goto 999
  277  continue
       call msgout(io,lunit, 'Too many commands defined')
@@ -1269,7 +1285,13 @@ c
 c---------------------------------------------------------------------
  998  continue
 c     message d'erreur soft
-      call msgout(io,lunit,buf(1:80))
+c Bug 1422 corrected - Francois VOGEL June 2006
+      bufl=1
+      do while ( buf(bufl:bufl).ne.char(0) .and. bufl.lt.80 )
+         bufl=bufl+1
+      enddo
+      bufl=bufl-1
+      call msgout(io,lunit,buf(1:bufl))
 c
  999  return
       end
@@ -1301,6 +1323,7 @@ c
       subroutine depfun(lunit,trace,first)
 c     depile une macro ou un execstr
 c Bug 1091 corrected - Francois VOGEL November 2004
+c Bug 1417 corrected - Francois VOGEL May 2005
       include '../stack.h'
       integer lunit
       logical trace,first,callback
@@ -1340,7 +1363,6 @@ c
             if(first) then
                buf='at line '
                m=11
-               first=.false.
                nlc=0
                call linestore(lct(8))
             else
@@ -1354,7 +1376,10 @@ c
             m=m+13
             if (km.le.isiz) then
                call cvnamel(idstk(1,km),buf(m+1:m+nlgh),1,leng)
-               call funnamestore(buf(m+1:m+nlgh),leng) 
+               if(first) then
+                  call funnamestore(buf(m+1:m+nlgh),leng)
+                  first=.false.
+               endif
                m=m+leng
             endif
          else
@@ -1384,8 +1409,11 @@ c
                call cvstr(m,lin(l1),buf(1:m),1)
                call basout(io,lunit,buf(1:m))
                if(istk(ilk).eq.10) then
-                  if(m.gt.24) m=24
-                  call funnamestore(buf(1:m),m) 
+                  if(first) then
+                     if(m.gt.24) m=24
+                     call funnamestore(buf(1:m),m) 
+                     first=.false.
+                  endif
                endif
             endif
          endif
@@ -1399,6 +1427,7 @@ c
       subroutine depexec(lunit,trace,first,pflag)
 c     pflag ,indique si c'est une pause qui a ete depilee
 c Bug 1091 corrected - Francois VOGEL November 2004
+c Bug 1417 corrected - Francois VOGEL May 2005
       include '../stack.h'
       logical trace,first,pflag
       integer mode(2),lunit,ll
@@ -1415,7 +1444,6 @@ c
             if(first) then
                buf='at line '
                m=11
-               first=.false.
                nlc=0
                call linestore(lct(8))
             else
@@ -1437,8 +1465,11 @@ c
             endif
             call cvstr(m,lin(l1),buf,1)
             call basout(io,lunit,buf(1:m))
-            if(m.gt.24) m=24
-            call funnamestore(buf(1:m),m) 
+            if(first) then
+               if(m.gt.24) m=24
+               call funnamestore(buf(1:m),m) 
+               first=.false.
+            endif
          endif
          mode(1)=0
          call clunit(-rio,buf,mode)
@@ -1461,11 +1492,10 @@ c     current statement
 c!
       include '../stack.h'
 
-      integer sadr
+c      integer sadr
 
-c
-      character mg*9,bel*1,line*340
-      integer n,ll,m,m1
+      character mg*9,bel*1,line*340,fmt*7
+      integer n,ll,m,m1,lp,nd
       data mg /' !--error'/,bel/' '/
 c
 c      sadr(l)=(l/2)+1
@@ -1492,9 +1522,13 @@ c
  1000 line=' '
       if(m1+15.gt.ll) m1=ll-16
       line(m1+1:m1+9)=mg
-      write(line(m1+11:m1+15),'(i5)') n
-      line(m1+16:m1+16)=bel
-      call basout(io,lunit,line(1:m1+16))
+      lp=m1+11
+      nd=int(log10(float(n)))+1
+      write(fmt,'(''(i'',i2,'')'')') nd
+      write(line(lp:lp+nd-1),fmt) n
+      lp=lp+nd
+      line(lp:lp)=bel
+      call basout(io,lunit,line(1:lp))
       return
       end
 
