@@ -7,8 +7,9 @@ c
       double precision val
       integer eol
       logical eqid,ptover,vargin,vargout,exec,vcopyobj
-      integer blank,r,ival(2),ptr,top1,count,iadr,wmacn
-      integer varargin(nsiz),varargout(nsiz)
+      integer blank,r,ival(2),ptr,top1,count,iadr,wmacn,p
+      integer varargin(nsiz),varargout(nsiz),id(nsiz)
+      integer scivarindex
       equivalence (ival(1),val)
       data blank/40/,eol/99/
       data varargin/169544223,387059739,nz2*673720360/
@@ -62,17 +63,19 @@ c     a macro
 c     .        see run to explain the following
                lcn=pstk(pt)-4-(nsiz+3)+1
                call putid(id,istk(lcn))
+            elseif(rstk(pt).eq.707) then
+c     .        function without argument list
+               call putid(id,syn)
             else
                call putid(id,ids(1,pt))
             endif
-            do 15 im=1,nmacs
-               if(eqid(id,macnms(1,im))) then
-                  wmacn=im
-                  goto 16
-               endif
- 15         continue
- 16         continue
+        endif
+
+         if(comp(1).eq.0) then
+c     .     find the index of the macro in the stack et memorize it for breakpoints
+            wmacn=scivarindex(fin)
          endif
+
          l=ilk+1
          if(exec) then
             vargout=.false.
@@ -242,11 +245,20 @@ c-----------------------------------------
 c     handle errcatch
       exec=rstk(pt-1).eq.909.or.rstk(pt-1).eq.903
       if(errct.ne.0.and.errpt.ge.pt.and..not.exec) then
-         errct=0
-         errpt=0
-         err1=0
+c     .  looking for a previous execstr(....,'errcatch') see intexecstr
+         p=pt
+ 401     if (rstk(p).eq.903)  goto 402
+         p=p-1
+         if (p.gt.0) goto 401
+ 402     err1=0
          err2=0
          catch=0
+         errct=0
+         errpt=0
+         if (p.gt.0) then
+            if (ids(1,p).eq.1) errct=-900001
+            errpt=p
+         endif
       endif
 
       lhsr=lhs
@@ -455,12 +467,14 @@ c
       pstk(pt)=wmac
       
 c     preserve current error recovery modes
-c      ids(2,pt)=errct
-c      ids(3,pt)=err2
-c      ids(4,pt)=err1
-c      ids(5,pt)=errpt
       if(r.eq.701.or.r.eq.604) then 
 c     .  disable error recovery mode , for pause only
+c         print *,'macro ',err1,err2,errct,errpt
+
+         ids(2,pt)=errct
+         ids(3,pt)=err2
+         ids(4,pt)=err1
+         ids(5,pt)=errpt
          err1=0
          catch=0
          errct=-1
@@ -488,19 +502,52 @@ c     fin exec
       endif
       wmac=pstk(pt)
       r=rstk(pt-1)
-
+      if(r.eq.701.or.r.eq.604) then 
 c     restore current error recovery modes
-c      print *,'macro ',top,err,err1,err2,errct,ids(2,pt),ids(3,pt),
-c     $     ids(4,pt),ids(5,pt),pt
-
-c      errct=ids(2,pt)
-c      err2=ids(3,pt)
-c      if (rstk(pt-1).ne.902) err1=ids(4,pt)
-c     errpt=ids(5,pt)
-
+c         print *,'macro ',err1,err2,errct,errpt
+         errct=ids(2,pt)
+         err2=ids(3,pt)
+         err1=ids(4,pt)
+         errpt=ids(5,pt)
+      endif
       pt=pt-1
       go to 99
 c     
  99   continue
       return
       end
+
+      integer function scivarindex(l)
+      include '../stack.h'
+      integer k0,k1,k2,l
+
+      if(l.ge.lstk(bot)) then
+         k0=bot
+         k2=isiz
+      elseif(l.le.lstk(top+1)) then
+         k0=1
+         k2=top+1
+      else
+         scivarindex=0
+         return
+      endif
+
+c     dichotomy search
+ 10   continue
+      if (k2-k0.le.1) goto 20
+      k1=int((k2-k0)/2)+k0
+      if(l.ge.lstk(k1)) then
+         k0=k1
+      else
+         k2=k1-1
+      endif
+      goto 10
+
+ 20   if (l.ge.lstk(k2)) then
+         scivarindex=k2
+      else
+         scivarindex=k0
+      endif
+      return
+      end
+
