@@ -7,6 +7,11 @@
 
 #include "intmacr2tree.h"
 
+#ifdef WIN32
+#include "../os_specific/win_mem_alloc.h"  /* malloc */
+#else
+#include "../os_specific/sci_mem_alloc.h"  /* malloc */
+#endif
 /* Table to store variable names */
 static char varnames[isizt][nlgh+1];
 
@@ -91,12 +96,12 @@ int C2F(macr2tree) _PARAMS((char *fname))
     }
 
   /* Memory allocation */
-  if((name=calloc(1,sizeof(char)))==NULL)
+  if((name=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"macr2tree: No more memory available\r\n");
       return 0;
     }
-  if((name[0]=(char *)calloc(1,sizeof(char)*(nlgh+1)))==NULL)
+  if((name[0]=(char *)CALLOC(1,sizeof(char)*(nlgh+1)))==NULL)
     {
       Scierror(999,"macr2tree: No more memory available\r\n");
       return 0;
@@ -146,7 +151,7 @@ int C2F(macr2tree) _PARAMS((char *fname))
 
   /* Make a copy variable passed as reference */
   /* Memory allocation */
-  if((data=(int *)calloc(1,sizeof(int)*(codelength+ilt+1)))==NULL)
+  if((data=(int *)CALLOC(1,sizeof(int)*(codelength+ilt+1)))==NULL)
     {
       Scierror(999,"macr2tree: No more memory available\r\n");
       return 0;
@@ -176,11 +181,11 @@ int C2F(macr2tree) _PARAMS((char *fname))
     Scierror(17,"macr2tree: stack size exceeded (Use stacksize function to increase it)\r\n");
 
     /* Free memory */
-    free(name[0]);
+    FREE(name[0]);
     name[0]=NULL;
-    free(name);
+    FREE(name);
     name=NULL;
-    free(data);
+    FREE(data);
     
     return 0;
   }
@@ -199,11 +204,11 @@ int C2F(macr2tree) _PARAMS((char *fname))
 	  if (Err>0 || C2F(errgst).err1>0)
 	    {
 	      /* Free memory */
-	      free(name[0]);
+	      FREE(name[0]);
 	      name[0]=NULL;
-	      free(name);
+	      FREE(name);
 	      name=NULL;
-	      free(data);
+	      FREE(data);
 
 	      return 0;
 	    }
@@ -228,11 +233,11 @@ int C2F(macr2tree) _PARAMS((char *fname))
 	      Scierror(999,"macr2tree: Out of code\r\n");
  
 	      /* Free memory */
-	      free(name[0]);
+	      FREE(name[0]);
 	      name[0]=NULL;
-	      free(name);
+	      FREE(name);
 	      name=NULL;
-	      free(data);
+	      FREE(data);
 	      
 	      return 0;
 	    }
@@ -242,11 +247,11 @@ int C2F(macr2tree) _PARAMS((char *fname))
 	Scierror(999,"macr2tree: wrong Top value %d instead of %d\r\n",Top,TopSave+1);
 
 	/* Free memory */
-	free(name[0]);
+	FREE(name[0]);
 	name[0]=NULL;
-	free(name);
+	FREE(name);
 	name=NULL;
-	free(data);
+	FREE(data);
 	
 	return 0;
       }
@@ -266,11 +271,11 @@ int C2F(macr2tree) _PARAMS((char *fname))
   C2F(mktlist)(&n_pgrm_tlist);
 
   /* Free memory */
-  free(name[0]);
+  FREE(name[0]);
   name[0]=NULL;
-  free(name);
+  FREE(name);
   name=NULL;
-  free(data);
+  FREE(data);
   
   return 0;
 }
@@ -311,12 +316,12 @@ static int CreateEOLList(void)
   int one = 1;
 
   /* Memory allocation */
-  if((eol=calloc(1,sizeof(char)))==NULL)
+  if((eol=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"CreateEOLList: No more memory available\r\n");
       return 0;
     }
-  if((eol[0]=(char *)calloc(1,sizeof(char)*(strlen("EOL")+1)))==NULL)
+  if((eol[0]=(char *)CALLOC(1,sizeof(char)*(strlen("EOL")+1)))==NULL)
     {
       Scierror(999,"CreateEOLList: No more memory available\r\n");
       return 0;
@@ -331,9 +336,9 @@ static int CreateEOLList(void)
   C2F(mklist)(&one);
 
   /* Free memory */
-  free(eol[0]);
+  FREE(eol[0]);
   eol[0]=NULL;
-  free(eol);
+  FREE(eol);
   eol=NULL;
 
   return 0;
@@ -398,12 +403,12 @@ static int GetInstruction(int *data,int *index,int *nblines,int *addinstr)
   *addinstr=0;
 
   /* Memory allocation */
-  if((name=calloc(1,sizeof(char)))==NULL)
+  if((name=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"GetInstruction: No more memory available\r\n");
       return 0;
     }
-  if((name[0]=(char *)calloc(1,sizeof(char)*(nlgh+1)))==NULL)
+  if((name[0]=(char *)CALLOC(1,sizeof(char)*(nlgh+1)))==NULL)
     {
       Scierror(999,"GetInstruction: No more memory available\r\n");
       return 0;
@@ -465,6 +470,11 @@ static int GetInstruction(int *data,int *index,int *nblines,int *addinstr)
     CreateCsteTList("emptymatrix",data,index);
     break;
   case 5: /* Operations */
+    if(data[*index+2]==0) 
+      {
+        *index +=3;
+        break;
+      }
     CreateOperationTList(data,index);
     break;
   case 6: /* Number */
@@ -486,8 +496,9 @@ static int GetInstruction(int *data,int *index,int *nblines,int *addinstr)
     GetControlInstruction(data,index,nblines);
     *addinstr=1;
     break;
-  case 11: /* NOT AFFECTED */
-    Scierror(999,"GetInstruction: code %d not yet implemented\r\n",data[*index]);
+  case 11: /* 'try-catch' control instruction */
+    GetControlInstruction(data,index,nblines);
+    *addinstr=1;
     break;
   case 12: /* pause */
   case 13: /* break */
@@ -533,7 +544,8 @@ static int GetInstruction(int *data,int *index,int *nblines,int *addinstr)
     Scierror(999,"GetInstruction: code %d not yet implemented\r\n",data[*index]);
     break;
   case 25: /* Compute profiling data */
-    Scierror(999,"GetInstruction: code %d not yet implemented\r\n",data[*index]);
+ /* This code is ignored */
+    *index += 2;
     break;
   case 26: /* Vector of strings */
     Scierror(999,"GetInstruction: code %d not yet implemented\r\n",data[*index]);
@@ -577,9 +589,9 @@ static int GetInstruction(int *data,int *index,int *nblines,int *addinstr)
   }
   
   /* Free memory */
-  free(name[0]);
+  FREE(name[0]);
   name[0]=NULL;
-  free(name);
+  FREE(name);
   name=NULL;
 
   return 0;
@@ -590,6 +602,11 @@ static int GetInstruction(int *data,int *index,int *nblines,int *addinstr)
 ****************************************************************/
 static int GetControlInstruction(int *data,int *index,int *nblines)
 {
+  /* try-catch */
+  char *trycatch_tlist[] = {"trycatch","trystat","catchstat"};
+  int m_trycatch_tlist = 1;
+  int n_trycatch_tlist = 3;
+  
   /* if */
   char *if_tlist[] = {"ifthenelse","expression","then","elseifs","else"};
   int m_if_tlist = 1;
@@ -652,12 +669,12 @@ static int GetControlInstruction(int *data,int *index,int *nblines)
 
       /* Get loop variable */
       /* Memory allocation */
-      if((name=calloc(1,sizeof(char)))==NULL)
+      if((name=CALLOC(1,sizeof(char)))==NULL)
 	{
 	  Scierror(999,"GetControlInstruction: No more memory available\r\n");
 	  return 0;
 	}
-      if((name[0]=(char *)calloc(1,sizeof(char)*(nlgh+1)))==NULL)
+      if((name[0]=(char *)CALLOC(1,sizeof(char)*(nlgh+1)))==NULL)
 	{
 	  Scierror(999,"GetControlInstruction: No more memory available\r\n");
 	  return 0;
@@ -691,11 +708,59 @@ static int GetControlInstruction(int *data,int *index,int *nblines)
       C2F(mktlist)(&n_for_tlist);
 
       /* Free memory */
-      free(name[0]);
+      FREE(name[0]);
       name[0]=NULL;
-      free(name);
+      FREE(name);
       name=NULL;
     }
+    /* TRYCATCH */
+  else if(data[*index]==11)
+    {  
+      index0 = *index;
+      
+      str2sci(trycatch_tlist,m_trycatch_tlist,n_trycatch_tlist);
+      
+      /* index now point to first code to use as an instruction code */
+      *index += 3;
+      
+      codelgth = data[index0+1];
+      endindex = *index + codelgth - 1;
+      
+      TopSave = Top;
+      /* Get try instructions */
+      while(*index<=endindex)
+	{
+	  GetInstruction(data,index,nblines,&newinstr);
+	  (*index)++;
+	}
+	
+      nbinstr = Top - TopSave;
+  
+      /* Create list of try instructions */
+      C2F(mklist)(&nbinstr);
+      last_eol_pos = -10; 
+      
+      codelgth = data[index0+2];
+      endindex = *index + codelgth - 1;
+      
+      TopSave = Top;
+      /* Get catch instructions */
+      while(*index<=endindex)
+	{
+	  GetInstruction(data,index,nblines,&newinstr);
+	  (*index)++;
+	}  
+      
+      nbinstr = Top - TopSave;
+  
+      /* Create list of catch instructions */
+      C2F(mklist)(&nbinstr);
+      
+     (*index)--;
+      
+      /* Create trycatch tlist */
+      C2F(mktlist)(&n_trycatch_tlist);
+    }
   /* IF - WHILE - SELECT */
   else
     {
@@ -913,17 +978,17 @@ static int CreateCsteTList(char *type,int *data,int *index)
       strlgth = data[*index];
       
       /* Memory allocation */
-      if((str=calloc(1,sizeof(char)))==NULL)
+      if((str=CALLOC(1,sizeof(char)))==NULL)
 	{
 	  Scierror(999,"CreateCsteTList: No more memory available\r\n");
 	  return 0;
 	}
-      if((str[0]=(char *)calloc(1,sizeof(char)*(strlgth+1)))==NULL)
+      if((str[0]=(char *)CALLOC(1,sizeof(char)*(strlgth+1)))==NULL)
 	{
 	  Scierror(999,"CreateCsteTList: No more memory available\r\n");
 	  return 0;
 	}
-      if((int_str=(int *)calloc(1,sizeof(int)*(strlgth+1)))==NULL)
+      if((int_str=(int *)CALLOC(1,sizeof(int)*(strlgth+1)))==NULL)
 	{
 	  Scierror(999,"CreateCsteTList: No more memory available\r\n");
 	  return 0;
@@ -939,11 +1004,11 @@ static int CreateCsteTList(char *type,int *data,int *index)
       str2sci(str,one,one);
 
       /* Free memory */
-      free(str[0]);
+      FREE(str[0]);
       str[0]=NULL;
-      free(str);
+      FREE(str);
       str=NULL;
-      free(int_str);
+      FREE(int_str);
       int_str=NULL;
     }
 
@@ -951,12 +1016,12 @@ static int CreateCsteTList(char *type,int *data,int *index)
     {
       strlgth=nlgh;
       /* Memory allocation */
-      if((str=calloc(1,sizeof(char)))==NULL)
+      if((str=CALLOC(1,sizeof(char)))==NULL)
 	{
 	  Scierror(999,"CreateCsteTList: No more memory available\r\n");
 	  return 0;
 	}
-      if((str[0]=(char *)calloc(1,sizeof(char)*(strlgth+1)))==NULL)
+      if((str[0]=(char *)CALLOC(1,sizeof(char)*(strlgth+1)))==NULL)
 	{
 	  Scierror(999,"CreateCsteTList: No more memory available\r\n");
 	  return 0;
@@ -971,15 +1036,15 @@ static int CreateCsteTList(char *type,int *data,int *index)
       str2sci(str,one,one);
 
       /* Free memory */
-      free(str[0]);
+      FREE(str[0]);
       str[0]=NULL;
-      free(str);
+      FREE(str);
       str=NULL;
     }
   else if(!strncmp(type,"number",6))
     {
       /* Memory allocation */
-      if((value=(double *)calloc(1,sizeof(double)))==NULL)
+      if((value=(double *)CALLOC(1,sizeof(double)))==NULL)
 	{
 	  Scierror(999,"CreateCsteTList: No more memory available\r\n");
 	  return 0;
@@ -1009,7 +1074,7 @@ static int CreateCsteTList(char *type,int *data,int *index)
       C2F(dtosci)(value,&one,&one);
       
       /* Free memory */
-      free(value);
+      FREE(value);
       value=NULL;
     }
   else /* Should never happen */
@@ -1057,12 +1122,12 @@ static int CreateOperationTList(int *data,int *index)
   int one = 1;
 
   /* Memory allocation */
-  if((operator=calloc(1,sizeof(char)))==NULL)
+  if((operator=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"CreateOperationTList: No more memory available\r\n");
       return 0;
     }
-  if((operator[0]=(char *)calloc(1,sizeof(char)*max_op_lgth+1))==NULL)
+  if((operator[0]=(char *)CALLOC(1,sizeof(char)*max_op_lgth+1))==NULL)
     {
       Scierror(999,"CreateOperationTList: No more memory available\r\n");
       return 0;
@@ -1189,12 +1254,12 @@ static int CreateFuncallTList(char *fromwhat,int *data,int *index)
   int n_mat = 0;
 
   /* Memory allocation */
-  if((funname=calloc(1,sizeof(char)))==NULL)
+  if((funname=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"CreateFuncallTList: No more memory available\r\n");
       return 0;
     }
-  if((funname[0]=(char *)calloc(1,sizeof(char)*(nlgh+1)))==NULL)
+  if((funname[0]=(char *)CALLOC(1,sizeof(char)*(nlgh+1)))==NULL)
     {
       Scierror(999,"CreateFuncallTList: No more memory available\r\n");
       return 0;
@@ -1312,9 +1377,9 @@ static int CreateFuncallTList(char *fromwhat,int *data,int *index)
   VCopyObj("CreateFuncallTList",&orig,&dest,18L);
 
   /* Free memory */
-  free(funname[0]);
+  FREE(funname[0]);
   funname[0]=NULL;
-  free(funname);
+  FREE(funname);
   funname=NULL;
 
   return 0;
@@ -1355,24 +1420,24 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
   int symbol = 0;
 
   /* Memory allocation */
-  if((name=calloc(1,sizeof(char)))==NULL)
+  if((name=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"CreateEqualTList: No more memory available\r\n");
       return 0;
     }
-  if((name[0]=(char *)calloc(1,sizeof(char)*(nlgh+1)))==NULL)
+  if((name[0]=(char *)CALLOC(1,sizeof(char)*(nlgh+1)))==NULL)
     {
       Scierror(999,"CreateEqualTList: No more memory available\r\n");
       return 0;
     }
   (name[0])[nlgh] = '\0';
 
-  if((operator=calloc(1,sizeof(char)))==NULL)
+  if((operator=CALLOC(1,sizeof(char)))==NULL)
     {
       Scierror(999,"CreateEqualTList: No more memory available\r\n");
       return 0;
     }
-  if((operator[0]=(char *)calloc(1,sizeof(char)*4))==NULL)
+  if((operator[0]=(char *)CALLOC(1,sizeof(char)*4))==NULL)
     {
       Scierror(999,"CreateEqualTList: No more memory available\r\n");
       return 0;
@@ -1401,12 +1466,12 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
       symbol=data[*index];
       if(symbol==43) /* ; */
 	{
-	  if((endsymbol=calloc(1,sizeof(char)))==NULL)
+	  if((endsymbol=CALLOC(1,sizeof(char)))==NULL)
 	    {
 	      Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	      return 0;
 	    }
-	  if((endsymbol[0]=(char *)calloc(1,sizeof(char)*2))==NULL)
+	  if((endsymbol[0]=(char *)CALLOC(1,sizeof(char)*2))==NULL)
 	    {
 	      Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	      return 0;
@@ -1416,12 +1481,12 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
 	}
       else if(symbol==52) /* , */
 	{
-	  if((endsymbol=calloc(1,sizeof(char)))==NULL)
+	  if((endsymbol=CALLOC(1,sizeof(char)))==NULL)
 	    {
 	      Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	      return 0;
 	    }
-	  if((endsymbol[0]=(char *)calloc(1,sizeof(char)*2))==NULL)
+	  if((endsymbol[0]=(char *)CALLOC(1,sizeof(char)*2))==NULL)
 	    {
 	      Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	      return 0;
@@ -1431,12 +1496,12 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
 	}
       else /* Nothing */
 	{
-	  if((endsymbol=calloc(1,sizeof(char)))==NULL)
+	  if((endsymbol=CALLOC(1,sizeof(char)))==NULL)
 	    {
 	      Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	      return 0;
 	    }
-	  if((endsymbol[0]=(char *)calloc(1,sizeof(char)*1))==NULL)
+	  if((endsymbol[0]=(char *)CALLOC(1,sizeof(char)*1))==NULL)
 	    {
 	      Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	      return 0;
@@ -1530,12 +1595,12 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
       C2F(mklist)(&nblhs);
 
       /* Symbol */
-      if((endsymbol=calloc(1,sizeof(char)))==NULL)
+      if((endsymbol=CALLOC(1,sizeof(char)))==NULL)
 	{
 	  Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	  return 0;
 	}
-      if((endsymbol[0]=(char *)calloc(1,sizeof(char)*1))==NULL)
+      if((endsymbol[0]=(char *)CALLOC(1,sizeof(char)*1))==NULL)
 	{
 	  Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	  return 0;
@@ -1573,12 +1638,12 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
       C2F(mklist)(&nblhs);
 
       /* Symbol */
-      if((endsymbol=calloc(1,sizeof(char)))==NULL)
+      if((endsymbol=CALLOC(1,sizeof(char)))==NULL)
 	{
 	  Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	  return 0;
 	}
-      if((endsymbol[0]=(char *)calloc(1,sizeof(char)*1))==NULL)
+      if((endsymbol[0]=(char *)CALLOC(1,sizeof(char)*1))==NULL)
 	{
 	  Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	  return 0;
@@ -1606,12 +1671,12 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
       C2F(mklist)(&nblhs);
 
       /* Symbol */
-      if((endsymbol=calloc(1,sizeof(char)))==NULL)
+      if((endsymbol=CALLOC(1,sizeof(char)))==NULL)
 	{
 	  Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	  return 0;
 	}
-      if((endsymbol[0]=(char *)calloc(1,sizeof(char)*1))==NULL)
+      if((endsymbol[0]=(char *)CALLOC(1,sizeof(char)*1))==NULL)
 	{
 	  Scierror(999,"CreateEqualTList: No more memory available\r\n");
 	  return 0;
@@ -1634,17 +1699,17 @@ static int CreateEqualTList(char *fromwhat,int *data,int *index)
     }
 
   /* Free memory */
-  free(name[0]);
+  FREE(name[0]);
   name[0]=NULL;
-  free(name);
+  FREE(name);
   name=NULL;
-  free(operator[0]);
+  FREE(operator[0]);
   operator[0]=NULL;
-  free(operator);
+  FREE(operator);
   operator=NULL;
-  free(endsymbol[0]);
+  FREE(endsymbol[0]);
   endsymbol[0]=NULL;
-  free(endsymbol);
+  FREE(endsymbol);
   endsymbol=0;
 
   return 0;
@@ -1673,7 +1738,7 @@ static int CreateCommentTList(int *data,int *index)
   strlgth = data[*index];
   (*index)++;   
   /* Memory allocation */
-  if((text=(char *)calloc(1,sizeof(char)*(strlgth+1)))==NULL)
+  if((text=(char *)CALLOC(1,sizeof(char)*(strlgth+1)))==NULL)
     {
       Scierror(999,"CreateCsteTList: No more memory available\r\n");
       return 0;
@@ -1683,7 +1748,7 @@ static int CreateCommentTList(int *data,int *index)
   str2sci(&text,one,one);
   *index = *index + strlgth-1;
   /* Free memory */
-  free(text);
+  FREE(text);
   text=NULL;
 
   C2F(mktlist)(&n_fun_tlist);
@@ -1813,7 +1878,9 @@ int complexity(int *data,int *index,int *lgth)
 	  cur_ind = cur_ind + data[cur_ind+1];
 	  count++;
 	  break;
-	case 11: /* NOT AFFECTED */
+	case 11: /* 'try-catch' control instruction */
+	  cur_ind = cur_ind + data[cur_ind+1] + data[cur_ind+2] + 3;
+	  count++;
 	  break;
 	case 12: /* pause */
 	  cur_ind++;
