@@ -1,8 +1,11 @@
 /* Allan CORNET */
 /* INRIA 2005 */
+/*-----------------------------------------------------------------------------------*/
 #ifndef __PRINTF_H__
 #define __PRINTF_H__
-
+/*-----------------------------------------------------------------------------------*/
+#include "../machine.h"
+/*-----------------------------------------------------------------------------------*/
 /* replacement stdio routines that use Text Window for stdin/stdout */
 /* WARNING: Do not write to stdout/stderr with functions not listed 
    in win/wtext.h */
@@ -30,32 +33,22 @@
 
 #undef fwrite
 #undef fread
-
+/*-----------------------------------------------------------------------------------*/
 int MyPutCh (int ch);
-int MyKBHit (void);
 int MyGetCh (void);
-int MyGetChE (void);
-int MyFGetC (FILE * file);
-char *MyGetS (char *str);
 char * MyFGetS (char *str, unsigned int size, FILE * file);
 int MyFPutC (int ch, FILE * file);
 int MyFPutS (char *str, FILE * file);
-int MyPutS (char *str);
 void Scistring (char *str);
-int MyFPrintF (FILE * file, char *fmt,...);
-int MyPrintF (char *fmt,...);
 void sciprint (char *fmt,...);
 void sciprint_nd (char *fmt,...);
 int sciprint2 (int iv, char *fmt,...);
 size_t MyFWrite (const void *ptr, size_t size, size_t n, FILE * file);
 size_t MyFRead (void *ptr, size_t size, size_t n, FILE * file);
-extern void Xputchar ();
 void Xputchar (c);
 void Xputstring (str, n);
 void Scisncr (str);
-int ReadKey (void);
-int WriteStr (char *s, FILE * file);
-int WriteKey (char c, FILE * file);
-
-
-#endif /* __PRINTF_H__ */
\ No newline at end of file
+void C2F(diary) __PARAMS((char *str,int *n));
+void diary_nnl __PARAMS((char *str,int *n));
+#endif /* __PRINTF_H__ */
+/*-----------------------------------------------------------------------------------*/
